/****************************************************************************
** Meta object code from reading C++ file 'qwaylandxdgshellv5.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/extensions/qwaylandxdgshellv5.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandxdgshellv5.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandXdgShellV5_t {
    QByteArrayData data[30];
    char stringdata0[370];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandXdgShellV5_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandXdgShellV5_t qt_meta_stringdata_QWaylandXdgShellV5 = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QWaylandXdgShellV5"
QT_MOC_LITERAL(1, 19, 19), // "xdgSurfaceRequested"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(4, 57, 7), // "surface"
QT_MOC_LITERAL(5, 65, 16), // "QWaylandResource"
QT_MOC_LITERAL(6, 82, 8), // "resource"
QT_MOC_LITERAL(7, 91, 17), // "xdgSurfaceCreated"
QT_MOC_LITERAL(8, 109, 21), // "QWaylandXdgSurfaceV5*"
QT_MOC_LITERAL(9, 131, 10), // "xdgSurface"
QT_MOC_LITERAL(10, 142, 15), // "xdgPopupCreated"
QT_MOC_LITERAL(11, 158, 19), // "QWaylandXdgPopupV5*"
QT_MOC_LITERAL(12, 178, 8), // "xdgPopup"
QT_MOC_LITERAL(13, 187, 17), // "xdgPopupRequested"
QT_MOC_LITERAL(14, 205, 6), // "parent"
QT_MOC_LITERAL(15, 212, 13), // "QWaylandSeat*"
QT_MOC_LITERAL(16, 226, 4), // "seat"
QT_MOC_LITERAL(17, 231, 8), // "position"
QT_MOC_LITERAL(18, 240, 4), // "pong"
QT_MOC_LITERAL(19, 245, 6), // "serial"
QT_MOC_LITERAL(20, 252, 4), // "ping"
QT_MOC_LITERAL(21, 257, 15), // "QWaylandClient*"
QT_MOC_LITERAL(22, 273, 6), // "client"
QT_MOC_LITERAL(23, 280, 14), // "closeAllPopups"
QT_MOC_LITERAL(24, 295, 17), // "handleSeatChanged"
QT_MOC_LITERAL(25, 313, 7), // "newSeat"
QT_MOC_LITERAL(26, 321, 7), // "oldSeat"
QT_MOC_LITERAL(27, 329, 18), // "handleFocusChanged"
QT_MOC_LITERAL(28, 348, 10), // "newSurface"
QT_MOC_LITERAL(29, 359, 10) // "oldSurface"

    },
    "QWaylandXdgShellV5\0xdgSurfaceRequested\0"
    "\0QWaylandSurface*\0surface\0QWaylandResource\0"
    "resource\0xdgSurfaceCreated\0"
    "QWaylandXdgSurfaceV5*\0xdgSurface\0"
    "xdgPopupCreated\0QWaylandXdgPopupV5*\0"
    "xdgPopup\0xdgPopupRequested\0parent\0"
    "QWaylandSeat*\0seat\0position\0pong\0"
    "serial\0ping\0QWaylandClient*\0client\0"
    "closeAllPopups\0handleSeatChanged\0"
    "newSeat\0oldSeat\0handleFocusChanged\0"
    "newSurface\0oldSurface"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandXdgShellV5[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   59,    2, 0x06 /* Public */,
       7,    1,   64,    2, 0x06 /* Public */,
      10,    1,   67,    2, 0x06 /* Public */,
      13,    5,   70,    2, 0x06 /* Public */,
      18,    1,   81,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      20,    1,   84,    2, 0x0a /* Public */,
      23,    0,   87,    2, 0x0a /* Public */,
      24,    2,   88,    2, 0x08 /* Private */,
      27,    2,   93,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 11,   12,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 15, QMetaType::QPoint, 0x80000000 | 5,    4,   14,   16,   17,    6,
    QMetaType::Void, QMetaType::UInt,   19,

 // slots: parameters
    QMetaType::UInt, 0x80000000 | 21,   22,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15, 0x80000000 | 15,   25,   26,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,   28,   29,

       0        // eod
};

void QWaylandXdgShellV5::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandXdgShellV5 *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->xdgSurfaceRequested((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< const QWaylandResource(*)>(_a[2]))); break;
        case 1: _t->xdgSurfaceCreated((*reinterpret_cast< QWaylandXdgSurfaceV5*(*)>(_a[1]))); break;
        case 2: _t->xdgPopupCreated((*reinterpret_cast< QWaylandXdgPopupV5*(*)>(_a[1]))); break;
        case 3: _t->xdgPopupRequested((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2])),(*reinterpret_cast< QWaylandSeat*(*)>(_a[3])),(*reinterpret_cast< const QPoint(*)>(_a[4])),(*reinterpret_cast< const QWaylandResource(*)>(_a[5]))); break;
        case 4: _t->pong((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 5: { uint _r = _t->ping((*reinterpret_cast< QWaylandClient*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->closeAllPopups(); break;
        case 7: _t->handleSeatChanged((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< QWaylandSeat*(*)>(_a[2]))); break;
        case 8: _t->handleFocusChanged((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandResource >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgSurfaceV5* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgPopupV5* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandResource >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandXdgShellV5::*)(QWaylandSurface * , const QWaylandResource & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgShellV5::xdgSurfaceRequested)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgShellV5::*)(QWaylandXdgSurfaceV5 * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgShellV5::xdgSurfaceCreated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgShellV5::*)(QWaylandXdgPopupV5 * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgShellV5::xdgPopupCreated)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgShellV5::*)(QWaylandSurface * , QWaylandSurface * , QWaylandSeat * , const QPoint & , const QWaylandResource & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgShellV5::xdgPopupRequested)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgShellV5::*)(uint );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgShellV5::pong)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QWaylandXdgShellV5::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandShellTemplate<QWaylandXdgShellV5>::staticMetaObject>(),
    qt_meta_stringdata_QWaylandXdgShellV5.data,
    qt_meta_data_QWaylandXdgShellV5,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandXdgShellV5::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandXdgShellV5::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandXdgShellV5.stringdata0))
        return static_cast<void*>(this);
    return QWaylandShellTemplate<QWaylandXdgShellV5>::qt_metacast(_clname);
}

int QWaylandXdgShellV5::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandShellTemplate<QWaylandXdgShellV5>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void QWaylandXdgShellV5::xdgSurfaceRequested(QWaylandSurface * _t1, const QWaylandResource & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QWaylandXdgShellV5::xdgSurfaceCreated(QWaylandXdgSurfaceV5 * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QWaylandXdgShellV5::xdgPopupCreated(QWaylandXdgPopupV5 * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QWaylandXdgShellV5::xdgPopupRequested(QWaylandSurface * _t1, QWaylandSurface * _t2, QWaylandSeat * _t3, const QPoint & _t4, const QWaylandResource & _t5)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QWaylandXdgShellV5::pong(uint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
struct qt_meta_stringdata_QWaylandXdgSurfaceV5_t {
    QByteArrayData data[83];
    char stringdata0[1067];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandXdgSurfaceV5_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandXdgSurfaceV5_t qt_meta_stringdata_QWaylandXdgSurfaceV5 = {
    {
QT_MOC_LITERAL(0, 0, 20), // "QWaylandXdgSurfaceV5"
QT_MOC_LITERAL(1, 21, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 37, 4), // "data"
QT_MOC_LITERAL(3, 42, 12), // "shellChanged"
QT_MOC_LITERAL(4, 55, 0), // ""
QT_MOC_LITERAL(5, 56, 14), // "surfaceChanged"
QT_MOC_LITERAL(6, 71, 12), // "titleChanged"
QT_MOC_LITERAL(7, 84, 21), // "windowGeometryChanged"
QT_MOC_LITERAL(8, 106, 12), // "appIdChanged"
QT_MOC_LITERAL(9, 119, 20), // "parentSurfaceChanged"
QT_MOC_LITERAL(10, 140, 13), // "statesChanged"
QT_MOC_LITERAL(11, 154, 16), // "maximizedChanged"
QT_MOC_LITERAL(12, 171, 17), // "fullscreenChanged"
QT_MOC_LITERAL(13, 189, 15), // "resizingChanged"
QT_MOC_LITERAL(14, 205, 16), // "activatedChanged"
QT_MOC_LITERAL(15, 222, 14), // "showWindowMenu"
QT_MOC_LITERAL(16, 237, 13), // "QWaylandSeat*"
QT_MOC_LITERAL(17, 251, 4), // "seat"
QT_MOC_LITERAL(18, 256, 20), // "localSurfacePosition"
QT_MOC_LITERAL(19, 277, 9), // "startMove"
QT_MOC_LITERAL(20, 287, 11), // "startResize"
QT_MOC_LITERAL(21, 299, 10), // "ResizeEdge"
QT_MOC_LITERAL(22, 310, 5), // "edges"
QT_MOC_LITERAL(23, 316, 11), // "setTopLevel"
QT_MOC_LITERAL(24, 328, 12), // "setTransient"
QT_MOC_LITERAL(25, 341, 12), // "setMaximized"
QT_MOC_LITERAL(26, 354, 14), // "unsetMaximized"
QT_MOC_LITERAL(27, 369, 13), // "setFullscreen"
QT_MOC_LITERAL(28, 383, 15), // "QWaylandOutput*"
QT_MOC_LITERAL(29, 399, 6), // "output"
QT_MOC_LITERAL(30, 406, 15), // "unsetFullscreen"
QT_MOC_LITERAL(31, 422, 12), // "setMinimized"
QT_MOC_LITERAL(32, 435, 12), // "ackConfigure"
QT_MOC_LITERAL(33, 448, 6), // "serial"
QT_MOC_LITERAL(34, 455, 24), // "handleSurfaceSizeChanged"
QT_MOC_LITERAL(35, 480, 24), // "handleBufferScaleChanged"
QT_MOC_LITERAL(36, 505, 10), // "initialize"
QT_MOC_LITERAL(37, 516, 19), // "QWaylandXdgShellV5*"
QT_MOC_LITERAL(38, 536, 8), // "xdgShell"
QT_MOC_LITERAL(39, 545, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(40, 562, 7), // "surface"
QT_MOC_LITERAL(41, 570, 16), // "QWaylandResource"
QT_MOC_LITERAL(42, 587, 8), // "resource"
QT_MOC_LITERAL(43, 596, 13), // "sizeForResize"
QT_MOC_LITERAL(44, 610, 4), // "size"
QT_MOC_LITERAL(45, 615, 5), // "delta"
QT_MOC_LITERAL(46, 621, 4), // "edge"
QT_MOC_LITERAL(47, 626, 13), // "sendConfigure"
QT_MOC_LITERAL(48, 640, 13), // "QVector<uint>"
QT_MOC_LITERAL(49, 654, 6), // "states"
QT_MOC_LITERAL(50, 661, 14), // "QVector<State>"
QT_MOC_LITERAL(51, 676, 9), // "sendClose"
QT_MOC_LITERAL(52, 686, 13), // "sendMaximized"
QT_MOC_LITERAL(53, 700, 15), // "sendUnmaximized"
QT_MOC_LITERAL(54, 716, 14), // "sendFullscreen"
QT_MOC_LITERAL(55, 731, 12), // "sendResizing"
QT_MOC_LITERAL(56, 744, 7), // "maxSize"
QT_MOC_LITERAL(57, 752, 25), // "QQmlListProperty<QObject>"
QT_MOC_LITERAL(58, 778, 5), // "shell"
QT_MOC_LITERAL(59, 784, 13), // "parentSurface"
QT_MOC_LITERAL(60, 798, 21), // "QWaylandXdgSurfaceV5*"
QT_MOC_LITERAL(61, 820, 5), // "title"
QT_MOC_LITERAL(62, 826, 5), // "appId"
QT_MOC_LITERAL(63, 832, 14), // "windowGeometry"
QT_MOC_LITERAL(64, 847, 10), // "QList<int>"
QT_MOC_LITERAL(65, 858, 9), // "maximized"
QT_MOC_LITERAL(66, 868, 10), // "fullscreen"
QT_MOC_LITERAL(67, 879, 8), // "resizing"
QT_MOC_LITERAL(68, 888, 9), // "activated"
QT_MOC_LITERAL(69, 898, 5), // "State"
QT_MOC_LITERAL(70, 904, 14), // "MaximizedState"
QT_MOC_LITERAL(71, 919, 15), // "FullscreenState"
QT_MOC_LITERAL(72, 935, 13), // "ResizingState"
QT_MOC_LITERAL(73, 949, 14), // "ActivatedState"
QT_MOC_LITERAL(74, 964, 8), // "NoneEdge"
QT_MOC_LITERAL(75, 973, 7), // "TopEdge"
QT_MOC_LITERAL(76, 981, 10), // "BottomEdge"
QT_MOC_LITERAL(77, 992, 8), // "LeftEdge"
QT_MOC_LITERAL(78, 1001, 11), // "TopLeftEdge"
QT_MOC_LITERAL(79, 1013, 14), // "BottomLeftEdge"
QT_MOC_LITERAL(80, 1028, 9), // "RightEdge"
QT_MOC_LITERAL(81, 1038, 12), // "TopRightEdge"
QT_MOC_LITERAL(82, 1051, 15) // "BottomRightEdge"

    },
    "QWaylandXdgSurfaceV5\0DefaultProperty\0"
    "data\0shellChanged\0\0surfaceChanged\0"
    "titleChanged\0windowGeometryChanged\0"
    "appIdChanged\0parentSurfaceChanged\0"
    "statesChanged\0maximizedChanged\0"
    "fullscreenChanged\0resizingChanged\0"
    "activatedChanged\0showWindowMenu\0"
    "QWaylandSeat*\0seat\0localSurfacePosition\0"
    "startMove\0startResize\0ResizeEdge\0edges\0"
    "setTopLevel\0setTransient\0setMaximized\0"
    "unsetMaximized\0setFullscreen\0"
    "QWaylandOutput*\0output\0unsetFullscreen\0"
    "setMinimized\0ackConfigure\0serial\0"
    "handleSurfaceSizeChanged\0"
    "handleBufferScaleChanged\0initialize\0"
    "QWaylandXdgShellV5*\0xdgShell\0"
    "QWaylandSurface*\0surface\0QWaylandResource\0"
    "resource\0sizeForResize\0size\0delta\0"
    "edge\0sendConfigure\0QVector<uint>\0"
    "states\0QVector<State>\0sendClose\0"
    "sendMaximized\0sendUnmaximized\0"
    "sendFullscreen\0sendResizing\0maxSize\0"
    "QQmlListProperty<QObject>\0shell\0"
    "parentSurface\0QWaylandXdgSurfaceV5*\0"
    "title\0appId\0windowGeometry\0QList<int>\0"
    "maximized\0fullscreen\0resizing\0activated\0"
    "State\0MaximizedState\0FullscreenState\0"
    "ResizingState\0ActivatedState\0NoneEdge\0"
    "TopEdge\0BottomEdge\0LeftEdge\0TopLeftEdge\0"
    "BottomLeftEdge\0RightEdge\0TopRightEdge\0"
    "BottomRightEdge"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandXdgSurfaceV5[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      34,   16, // methods
      12,  262, // properties
       2,  310, // enums/sets
       0,    0, // constructors
       0,       // flags
      22,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  186,    4, 0x06 /* Public */,
       5,    0,  187,    4, 0x06 /* Public */,
       6,    0,  188,    4, 0x06 /* Public */,
       7,    0,  189,    4, 0x06 /* Public */,
       8,    0,  190,    4, 0x06 /* Public */,
       9,    0,  191,    4, 0x06 /* Public */,
      10,    0,  192,    4, 0x06 /* Public */,
      11,    0,  193,    4, 0x06 /* Public */,
      12,    0,  194,    4, 0x06 /* Public */,
      13,    0,  195,    4, 0x06 /* Public */,
      14,    0,  196,    4, 0x06 /* Public */,
      15,    2,  197,    4, 0x06 /* Public */,
      19,    1,  202,    4, 0x06 /* Public */,
      20,    2,  205,    4, 0x06 /* Public */,
      23,    0,  210,    4, 0x06 /* Public */,
      24,    0,  211,    4, 0x06 /* Public */,
      25,    0,  212,    4, 0x06 /* Public */,
      26,    0,  213,    4, 0x06 /* Public */,
      27,    1,  214,    4, 0x06 /* Public */,
      30,    0,  217,    4, 0x06 /* Public */,
      31,    0,  218,    4, 0x06 /* Public */,
      32,    1,  219,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      34,    0,  222,    4, 0x08 /* Private */,
      35,    0,  223,    4, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      36,    3,  224,    4, 0x02 /* Public */,
      43,    3,  231,    4, 0x02 /* Public */,
      47,    2,  238,    4, 0x02 /* Public */,
      47,    2,  243,    4, 0x02 /* Public */,
      51,    0,  248,    4, 0x02 /* Public */,
      52,    1,  249,    4, 0x02 /* Public */,
      53,    1,  252,    4, 0x02 /* Public */,
      53,    0,  255,    4, 0x22 /* Public | MethodCloned */,
      54,    1,  256,    4, 0x02 /* Public */,
      55,    1,  259,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16, QMetaType::QPoint,   17,   18,
    QMetaType::Void, 0x80000000 | 16,   17,
    QMetaType::Void, 0x80000000 | 16, 0x80000000 | 21,   17,   22,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UInt,   33,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 37, 0x80000000 | 39, 0x80000000 | 41,   38,   40,   42,
    QMetaType::QSize, QMetaType::QSizeF, QMetaType::QPointF, 0x80000000 | 21,   44,   45,   46,
    QMetaType::UInt, QMetaType::QSize, 0x80000000 | 48,   44,   49,
    QMetaType::UInt, QMetaType::QSize, 0x80000000 | 50,   44,   49,
    QMetaType::Void,
    QMetaType::UInt, QMetaType::QSize,   44,
    QMetaType::UInt, QMetaType::QSize,   44,
    QMetaType::UInt,
    QMetaType::UInt, QMetaType::QSize,   44,
    QMetaType::UInt, QMetaType::QSize,   56,

 // properties: name, type, flags
       2, 0x80000000 | 57, 0x00094009,
      58, 0x80000000 | 37, 0x00495009,
      40, 0x80000000 | 39, 0x00495009,
      59, 0x80000000 | 60, 0x00495009,
      61, QMetaType::QString, 0x00495001,
      62, QMetaType::QString, 0x00495001,
      63, QMetaType::QRect, 0x00495001,
      49, 0x80000000 | 64, 0x00495009,
      65, QMetaType::Bool, 0x00495001,
      66, QMetaType::Bool, 0x00495001,
      67, QMetaType::Bool, 0x00495001,
      68, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
       1,
       5,
       2,
       4,
       3,
       6,
       7,
       8,
       9,
      10,

 // enums: name, alias, flags, count, data
      69,   69, 0x0,    4,  320,
      21,   21, 0x0,    9,  328,

 // enum data: key, value
      70, uint(QWaylandXdgSurfaceV5::MaximizedState),
      71, uint(QWaylandXdgSurfaceV5::FullscreenState),
      72, uint(QWaylandXdgSurfaceV5::ResizingState),
      73, uint(QWaylandXdgSurfaceV5::ActivatedState),
      74, uint(QWaylandXdgSurfaceV5::NoneEdge),
      75, uint(QWaylandXdgSurfaceV5::TopEdge),
      76, uint(QWaylandXdgSurfaceV5::BottomEdge),
      77, uint(QWaylandXdgSurfaceV5::LeftEdge),
      78, uint(QWaylandXdgSurfaceV5::TopLeftEdge),
      79, uint(QWaylandXdgSurfaceV5::BottomLeftEdge),
      80, uint(QWaylandXdgSurfaceV5::RightEdge),
      81, uint(QWaylandXdgSurfaceV5::TopRightEdge),
      82, uint(QWaylandXdgSurfaceV5::BottomRightEdge),

       0        // eod
};

void QWaylandXdgSurfaceV5::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandXdgSurfaceV5 *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->shellChanged(); break;
        case 1: _t->surfaceChanged(); break;
        case 2: _t->titleChanged(); break;
        case 3: _t->windowGeometryChanged(); break;
        case 4: _t->appIdChanged(); break;
        case 5: _t->parentSurfaceChanged(); break;
        case 6: _t->statesChanged(); break;
        case 7: _t->maximizedChanged(); break;
        case 8: _t->fullscreenChanged(); break;
        case 9: _t->resizingChanged(); break;
        case 10: _t->activatedChanged(); break;
        case 11: _t->showWindowMenu((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2]))); break;
        case 12: _t->startMove((*reinterpret_cast< QWaylandSeat*(*)>(_a[1]))); break;
        case 13: _t->startResize((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< ResizeEdge(*)>(_a[2]))); break;
        case 14: _t->setTopLevel(); break;
        case 15: _t->setTransient(); break;
        case 16: _t->setMaximized(); break;
        case 17: _t->unsetMaximized(); break;
        case 18: _t->setFullscreen((*reinterpret_cast< QWaylandOutput*(*)>(_a[1]))); break;
        case 19: _t->unsetFullscreen(); break;
        case 20: _t->setMinimized(); break;
        case 21: _t->ackConfigure((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 22: _t->handleSurfaceSizeChanged(); break;
        case 23: _t->handleBufferScaleChanged(); break;
        case 24: _t->initialize((*reinterpret_cast< QWaylandXdgShellV5*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2])),(*reinterpret_cast< const QWaylandResource(*)>(_a[3]))); break;
        case 25: { QSize _r = _t->sizeForResize((*reinterpret_cast< const QSizeF(*)>(_a[1])),(*reinterpret_cast< const QPointF(*)>(_a[2])),(*reinterpret_cast< ResizeEdge(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QSize*>(_a[0]) = std::move(_r); }  break;
        case 26: { uint _r = _t->sendConfigure((*reinterpret_cast< const QSize(*)>(_a[1])),(*reinterpret_cast< const QVector<uint>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 27: { uint _r = _t->sendConfigure((*reinterpret_cast< const QSize(*)>(_a[1])),(*reinterpret_cast< const QVector<State>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->sendClose(); break;
        case 29: { uint _r = _t->sendMaximized((*reinterpret_cast< const QSize(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 30: { uint _r = _t->sendUnmaximized((*reinterpret_cast< const QSize(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 31: { uint _r = _t->sendUnmaximized();
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 32: { uint _r = _t->sendFullscreen((*reinterpret_cast< const QSize(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 33: { uint _r = _t->sendResizing((*reinterpret_cast< const QSize(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandResource >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgShellV5* >(); break;
            }
            break;
        case 26:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<uint> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::shellChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::surfaceChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::titleChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::windowGeometryChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::appIdChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::parentSurfaceChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::statesChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::maximizedChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::fullscreenChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::resizingChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::activatedChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)(QWaylandSeat * , const QPoint & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::showWindowMenu)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)(QWaylandSeat * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::startMove)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)(QWaylandSeat * , ResizeEdge );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::startResize)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::setTopLevel)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::setTransient)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::setMaximized)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::unsetMaximized)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)(QWaylandOutput * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::setFullscreen)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::unsetFullscreen)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::setMinimized)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurfaceV5::*)(uint );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurfaceV5::ackConfigure)) {
                *result = 21;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgShellV5* >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgSurfaceV5* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandXdgSurfaceV5 *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->data(); break;
        case 1: *reinterpret_cast< QWaylandXdgShellV5**>(_v) = _t->shell(); break;
        case 2: *reinterpret_cast< QWaylandSurface**>(_v) = _t->surface(); break;
        case 3: *reinterpret_cast< QWaylandXdgSurfaceV5**>(_v) = _t->parentSurface(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->title(); break;
        case 5: *reinterpret_cast< QString*>(_v) = _t->appId(); break;
        case 6: *reinterpret_cast< QRect*>(_v) = _t->windowGeometry(); break;
        case 7: *reinterpret_cast< QList<int>*>(_v) = _t->statesAsInts(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->maximized(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->fullscreen(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->resizing(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->activated(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandXdgSurfaceV5::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandShellSurfaceTemplate<QWaylandXdgSurfaceV5>::staticMetaObject>(),
    qt_meta_stringdata_QWaylandXdgSurfaceV5.data,
    qt_meta_data_QWaylandXdgSurfaceV5,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandXdgSurfaceV5::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandXdgSurfaceV5::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandXdgSurfaceV5.stringdata0))
        return static_cast<void*>(this);
    return QWaylandShellSurfaceTemplate<QWaylandXdgSurfaceV5>::qt_metacast(_clname);
}

int QWaylandXdgSurfaceV5::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandShellSurfaceTemplate<QWaylandXdgSurfaceV5>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 12;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandXdgSurfaceV5::shellChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandXdgSurfaceV5::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWaylandXdgSurfaceV5::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QWaylandXdgSurfaceV5::windowGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QWaylandXdgSurfaceV5::appIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QWaylandXdgSurfaceV5::parentSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QWaylandXdgSurfaceV5::statesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QWaylandXdgSurfaceV5::maximizedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QWaylandXdgSurfaceV5::fullscreenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QWaylandXdgSurfaceV5::resizingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QWaylandXdgSurfaceV5::activatedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QWaylandXdgSurfaceV5::showWindowMenu(QWaylandSeat * _t1, const QPoint & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QWaylandXdgSurfaceV5::startMove(QWaylandSeat * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QWaylandXdgSurfaceV5::startResize(QWaylandSeat * _t1, ResizeEdge _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void QWaylandXdgSurfaceV5::setTopLevel()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QWaylandXdgSurfaceV5::setTransient()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QWaylandXdgSurfaceV5::setMaximized()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QWaylandXdgSurfaceV5::unsetMaximized()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QWaylandXdgSurfaceV5::setFullscreen(QWaylandOutput * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void QWaylandXdgSurfaceV5::unsetFullscreen()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QWaylandXdgSurfaceV5::setMinimized()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void QWaylandXdgSurfaceV5::ackConfigure(uint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}
struct qt_meta_stringdata_QWaylandXdgPopupV5_t {
    QByteArrayData data[19];
    char stringdata0[257];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandXdgPopupV5_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandXdgPopupV5_t qt_meta_stringdata_QWaylandXdgPopupV5 = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QWaylandXdgPopupV5"
QT_MOC_LITERAL(1, 19, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 35, 4), // "data"
QT_MOC_LITERAL(3, 40, 12), // "shellChanged"
QT_MOC_LITERAL(4, 53, 0), // ""
QT_MOC_LITERAL(5, 54, 14), // "surfaceChanged"
QT_MOC_LITERAL(6, 69, 20), // "parentSurfaceChanged"
QT_MOC_LITERAL(7, 90, 15), // "positionChanged"
QT_MOC_LITERAL(8, 106, 10), // "initialize"
QT_MOC_LITERAL(9, 117, 19), // "QWaylandXdgShellV5*"
QT_MOC_LITERAL(10, 137, 5), // "shell"
QT_MOC_LITERAL(11, 143, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(12, 160, 7), // "surface"
QT_MOC_LITERAL(13, 168, 13), // "parentSurface"
QT_MOC_LITERAL(14, 182, 8), // "position"
QT_MOC_LITERAL(15, 191, 16), // "QWaylandResource"
QT_MOC_LITERAL(16, 208, 8), // "resource"
QT_MOC_LITERAL(17, 217, 13), // "sendPopupDone"
QT_MOC_LITERAL(18, 231, 25) // "QQmlListProperty<QObject>"

    },
    "QWaylandXdgPopupV5\0DefaultProperty\0"
    "data\0shellChanged\0\0surfaceChanged\0"
    "parentSurfaceChanged\0positionChanged\0"
    "initialize\0QWaylandXdgShellV5*\0shell\0"
    "QWaylandSurface*\0surface\0parentSurface\0"
    "position\0QWaylandResource\0resource\0"
    "sendPopupDone\0QQmlListProperty<QObject>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandXdgPopupV5[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       6,   16, // methods
       5,   62, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   46,    4, 0x06 /* Public */,
       5,    0,   47,    4, 0x06 /* Public */,
       6,    0,   48,    4, 0x06 /* Public */,
       7,    0,   49,    4, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       8,    5,   50,    4, 0x02 /* Public */,
      17,    0,   61,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 11, 0x80000000 | 11, QMetaType::QPoint, 0x80000000 | 15,   10,   12,   13,   14,   16,
    QMetaType::Void,

 // properties: name, type, flags
       2, 0x80000000 | 18, 0x00094009,
      10, 0x80000000 | 9, 0x00495009,
      12, 0x80000000 | 11, 0x00495009,
      13, 0x80000000 | 11, 0x00495009,
      14, QMetaType::QPoint, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
       1,
       2,
       3,

       0        // eod
};

void QWaylandXdgPopupV5::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandXdgPopupV5 *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->shellChanged(); break;
        case 1: _t->surfaceChanged(); break;
        case 2: _t->parentSurfaceChanged(); break;
        case 3: _t->positionChanged(); break;
        case 4: _t->initialize((*reinterpret_cast< QWaylandXdgShellV5*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[3])),(*reinterpret_cast< const QPoint(*)>(_a[4])),(*reinterpret_cast< const QWaylandResource(*)>(_a[5]))); break;
        case 5: _t->sendPopupDone(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 4:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandResource >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgShellV5* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandXdgPopupV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgPopupV5::shellChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgPopupV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgPopupV5::surfaceChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgPopupV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgPopupV5::parentSurfaceChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgPopupV5::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgPopupV5::positionChanged)) {
                *result = 3;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgShellV5* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandXdgPopupV5 *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->data(); break;
        case 1: *reinterpret_cast< QWaylandXdgShellV5**>(_v) = _t->shell(); break;
        case 2: *reinterpret_cast< QWaylandSurface**>(_v) = _t->surface(); break;
        case 3: *reinterpret_cast< QWaylandSurface**>(_v) = _t->parentSurface(); break;
        case 4: *reinterpret_cast< QPoint*>(_v) = _t->position(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandXdgPopupV5::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandShellSurfaceTemplate<QWaylandXdgPopupV5>::staticMetaObject>(),
    qt_meta_stringdata_QWaylandXdgPopupV5.data,
    qt_meta_data_QWaylandXdgPopupV5,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandXdgPopupV5::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandXdgPopupV5::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandXdgPopupV5.stringdata0))
        return static_cast<void*>(this);
    return QWaylandShellSurfaceTemplate<QWaylandXdgPopupV5>::qt_metacast(_clname);
}

int QWaylandXdgPopupV5::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandShellSurfaceTemplate<QWaylandXdgPopupV5>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandXdgPopupV5::shellChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandXdgPopupV5::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWaylandXdgPopupV5::parentSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QWaylandXdgPopupV5::positionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
