// This file was generated by qtwaylandscanner
// source file is /scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VckScFull/build/tmp/work/vck_sc_zynqmp-xilinx-linux/qtwayland/5.15.4+gitAUTOINC+f1e6c8764d-r0/git/src/3rdparty/protocol/xdg-shell-unstable-v6.xml

#include "qwayland-xdg-shell-unstable-v6.h"

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")

namespace QtWayland {

static inline void *wlRegistryBind(struct ::wl_registry *registry, uint32_t name, const struct ::wl_interface *interface, uint32_t version)
{
    const uint32_t bindOpCode = 0;
#if (WAYLAND_VERSION_MAJOR == 1 && WAYLAND_VERSION_MINOR > 10) || WAYLAND_VERSION_MAJOR > 1
    return (void *) wl_proxy_marshal_constructor_versioned((struct wl_proxy *) registry,
        bindOpCode, interface, version, name, interface->name, version, nullptr);
#else
    return (void *) wl_proxy_marshal_constructor((struct wl_proxy *) registry,
        bindOpCode, interface, name, interface->name, version, nullptr);
#endif
}

    zxdg_shell_v6::zxdg_shell_v6(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    zxdg_shell_v6::zxdg_shell_v6(struct ::zxdg_shell_v6 *obj)
        : m_zxdg_shell_v6(obj)
    {
        init_listener();
    }

    zxdg_shell_v6::zxdg_shell_v6()
        : m_zxdg_shell_v6(nullptr)
    {
    }

    zxdg_shell_v6::~zxdg_shell_v6()
    {
    }

    void zxdg_shell_v6::init(struct ::wl_registry *registry, int id, int version)
    {
        m_zxdg_shell_v6 = static_cast<struct ::zxdg_shell_v6 *>(wlRegistryBind(registry, id, &zxdg_shell_v6_interface, version));
        init_listener();
    }

    void zxdg_shell_v6::init(struct ::zxdg_shell_v6 *obj)
    {
        m_zxdg_shell_v6 = obj;
        init_listener();
    }

    zxdg_shell_v6 *zxdg_shell_v6::fromObject(struct ::zxdg_shell_v6 *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_zxdg_shell_v6_listener)
            return nullptr;
        return static_cast<zxdg_shell_v6 *>(zxdg_shell_v6_get_user_data(object));
    }

    bool zxdg_shell_v6::isInitialized() const
    {
        return m_zxdg_shell_v6 != nullptr;
    }

    const struct wl_interface *zxdg_shell_v6::interface()
    {
        return &::zxdg_shell_v6_interface;
    }

    void zxdg_shell_v6::destroy()
    {
        zxdg_shell_v6_destroy(
            m_zxdg_shell_v6);
        m_zxdg_shell_v6 = nullptr;
    }

    struct ::zxdg_positioner_v6 *zxdg_shell_v6::create_positioner()
    {
        return zxdg_shell_v6_create_positioner(
            m_zxdg_shell_v6);
    }

    struct ::zxdg_surface_v6 *zxdg_shell_v6::get_xdg_surface(struct ::wl_surface *surface)
    {
        return zxdg_shell_v6_get_xdg_surface(
            m_zxdg_shell_v6,
            surface);
    }

    void zxdg_shell_v6::pong(uint32_t serial)
    {
        zxdg_shell_v6_pong(
            m_zxdg_shell_v6,
            serial);
    }

    void zxdg_shell_v6::zxdg_shell_v6_ping(uint32_t )
    {
    }

    void zxdg_shell_v6::handle_ping(
        void *data,
        struct ::zxdg_shell_v6 *object,
        uint32_t serial)
    {
        Q_UNUSED(object);
        static_cast<zxdg_shell_v6 *>(data)->zxdg_shell_v6_ping(
            serial);
    }

    const struct zxdg_shell_v6_listener zxdg_shell_v6::m_zxdg_shell_v6_listener = {
        zxdg_shell_v6::handle_ping,
    };

    void zxdg_shell_v6::init_listener()
    {
        zxdg_shell_v6_add_listener(m_zxdg_shell_v6, &m_zxdg_shell_v6_listener, this);
    }

    zxdg_positioner_v6::zxdg_positioner_v6(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    zxdg_positioner_v6::zxdg_positioner_v6(struct ::zxdg_positioner_v6 *obj)
        : m_zxdg_positioner_v6(obj)
    {
    }

    zxdg_positioner_v6::zxdg_positioner_v6()
        : m_zxdg_positioner_v6(nullptr)
    {
    }

    zxdg_positioner_v6::~zxdg_positioner_v6()
    {
    }

    void zxdg_positioner_v6::init(struct ::wl_registry *registry, int id, int version)
    {
        m_zxdg_positioner_v6 = static_cast<struct ::zxdg_positioner_v6 *>(wlRegistryBind(registry, id, &zxdg_positioner_v6_interface, version));
    }

    void zxdg_positioner_v6::init(struct ::zxdg_positioner_v6 *obj)
    {
        m_zxdg_positioner_v6 = obj;
    }

    zxdg_positioner_v6 *zxdg_positioner_v6::fromObject(struct ::zxdg_positioner_v6 *object)
    {
        return static_cast<zxdg_positioner_v6 *>(zxdg_positioner_v6_get_user_data(object));
    }

    bool zxdg_positioner_v6::isInitialized() const
    {
        return m_zxdg_positioner_v6 != nullptr;
    }

    const struct wl_interface *zxdg_positioner_v6::interface()
    {
        return &::zxdg_positioner_v6_interface;
    }

    void zxdg_positioner_v6::destroy()
    {
        zxdg_positioner_v6_destroy(
            m_zxdg_positioner_v6);
        m_zxdg_positioner_v6 = nullptr;
    }

    void zxdg_positioner_v6::set_size(int32_t width, int32_t height)
    {
        zxdg_positioner_v6_set_size(
            m_zxdg_positioner_v6,
            width,
            height);
    }

    void zxdg_positioner_v6::set_anchor_rect(int32_t x, int32_t y, int32_t width, int32_t height)
    {
        zxdg_positioner_v6_set_anchor_rect(
            m_zxdg_positioner_v6,
            x,
            y,
            width,
            height);
    }

    void zxdg_positioner_v6::set_anchor(uint32_t anchor)
    {
        zxdg_positioner_v6_set_anchor(
            m_zxdg_positioner_v6,
            anchor);
    }

    void zxdg_positioner_v6::set_gravity(uint32_t gravity)
    {
        zxdg_positioner_v6_set_gravity(
            m_zxdg_positioner_v6,
            gravity);
    }

    void zxdg_positioner_v6::set_constraint_adjustment(uint32_t constraint_adjustment)
    {
        zxdg_positioner_v6_set_constraint_adjustment(
            m_zxdg_positioner_v6,
            constraint_adjustment);
    }

    void zxdg_positioner_v6::set_offset(int32_t x, int32_t y)
    {
        zxdg_positioner_v6_set_offset(
            m_zxdg_positioner_v6,
            x,
            y);
    }

    zxdg_surface_v6::zxdg_surface_v6(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    zxdg_surface_v6::zxdg_surface_v6(struct ::zxdg_surface_v6 *obj)
        : m_zxdg_surface_v6(obj)
    {
        init_listener();
    }

    zxdg_surface_v6::zxdg_surface_v6()
        : m_zxdg_surface_v6(nullptr)
    {
    }

    zxdg_surface_v6::~zxdg_surface_v6()
    {
    }

    void zxdg_surface_v6::init(struct ::wl_registry *registry, int id, int version)
    {
        m_zxdg_surface_v6 = static_cast<struct ::zxdg_surface_v6 *>(wlRegistryBind(registry, id, &zxdg_surface_v6_interface, version));
        init_listener();
    }

    void zxdg_surface_v6::init(struct ::zxdg_surface_v6 *obj)
    {
        m_zxdg_surface_v6 = obj;
        init_listener();
    }

    zxdg_surface_v6 *zxdg_surface_v6::fromObject(struct ::zxdg_surface_v6 *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_zxdg_surface_v6_listener)
            return nullptr;
        return static_cast<zxdg_surface_v6 *>(zxdg_surface_v6_get_user_data(object));
    }

    bool zxdg_surface_v6::isInitialized() const
    {
        return m_zxdg_surface_v6 != nullptr;
    }

    const struct wl_interface *zxdg_surface_v6::interface()
    {
        return &::zxdg_surface_v6_interface;
    }

    void zxdg_surface_v6::destroy()
    {
        zxdg_surface_v6_destroy(
            m_zxdg_surface_v6);
        m_zxdg_surface_v6 = nullptr;
    }

    struct ::zxdg_toplevel_v6 *zxdg_surface_v6::get_toplevel()
    {
        return zxdg_surface_v6_get_toplevel(
            m_zxdg_surface_v6);
    }

    struct ::zxdg_popup_v6 *zxdg_surface_v6::get_popup(struct ::zxdg_surface_v6 *parent, struct ::zxdg_positioner_v6 *positioner)
    {
        return zxdg_surface_v6_get_popup(
            m_zxdg_surface_v6,
            parent,
            positioner);
    }

    void zxdg_surface_v6::set_window_geometry(int32_t x, int32_t y, int32_t width, int32_t height)
    {
        zxdg_surface_v6_set_window_geometry(
            m_zxdg_surface_v6,
            x,
            y,
            width,
            height);
    }

    void zxdg_surface_v6::ack_configure(uint32_t serial)
    {
        zxdg_surface_v6_ack_configure(
            m_zxdg_surface_v6,
            serial);
    }

    void zxdg_surface_v6::zxdg_surface_v6_configure(uint32_t )
    {
    }

    void zxdg_surface_v6::handle_configure(
        void *data,
        struct ::zxdg_surface_v6 *object,
        uint32_t serial)
    {
        Q_UNUSED(object);
        static_cast<zxdg_surface_v6 *>(data)->zxdg_surface_v6_configure(
            serial);
    }

    const struct zxdg_surface_v6_listener zxdg_surface_v6::m_zxdg_surface_v6_listener = {
        zxdg_surface_v6::handle_configure,
    };

    void zxdg_surface_v6::init_listener()
    {
        zxdg_surface_v6_add_listener(m_zxdg_surface_v6, &m_zxdg_surface_v6_listener, this);
    }

    zxdg_toplevel_v6::zxdg_toplevel_v6(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    zxdg_toplevel_v6::zxdg_toplevel_v6(struct ::zxdg_toplevel_v6 *obj)
        : m_zxdg_toplevel_v6(obj)
    {
        init_listener();
    }

    zxdg_toplevel_v6::zxdg_toplevel_v6()
        : m_zxdg_toplevel_v6(nullptr)
    {
    }

    zxdg_toplevel_v6::~zxdg_toplevel_v6()
    {
    }

    void zxdg_toplevel_v6::init(struct ::wl_registry *registry, int id, int version)
    {
        m_zxdg_toplevel_v6 = static_cast<struct ::zxdg_toplevel_v6 *>(wlRegistryBind(registry, id, &zxdg_toplevel_v6_interface, version));
        init_listener();
    }

    void zxdg_toplevel_v6::init(struct ::zxdg_toplevel_v6 *obj)
    {
        m_zxdg_toplevel_v6 = obj;
        init_listener();
    }

    zxdg_toplevel_v6 *zxdg_toplevel_v6::fromObject(struct ::zxdg_toplevel_v6 *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_zxdg_toplevel_v6_listener)
            return nullptr;
        return static_cast<zxdg_toplevel_v6 *>(zxdg_toplevel_v6_get_user_data(object));
    }

    bool zxdg_toplevel_v6::isInitialized() const
    {
        return m_zxdg_toplevel_v6 != nullptr;
    }

    const struct wl_interface *zxdg_toplevel_v6::interface()
    {
        return &::zxdg_toplevel_v6_interface;
    }

    void zxdg_toplevel_v6::destroy()
    {
        zxdg_toplevel_v6_destroy(
            m_zxdg_toplevel_v6);
        m_zxdg_toplevel_v6 = nullptr;
    }

    void zxdg_toplevel_v6::set_parent(struct ::zxdg_toplevel_v6 *parent)
    {
        zxdg_toplevel_v6_set_parent(
            m_zxdg_toplevel_v6,
            parent);
    }

    void zxdg_toplevel_v6::set_title(const QString &title)
    {
        zxdg_toplevel_v6_set_title(
            m_zxdg_toplevel_v6,
            title.toUtf8().constData());
    }

    void zxdg_toplevel_v6::set_app_id(const QString &app_id)
    {
        zxdg_toplevel_v6_set_app_id(
            m_zxdg_toplevel_v6,
            app_id.toUtf8().constData());
    }

    void zxdg_toplevel_v6::show_window_menu(struct ::wl_seat *seat, uint32_t serial, int32_t x, int32_t y)
    {
        zxdg_toplevel_v6_show_window_menu(
            m_zxdg_toplevel_v6,
            seat,
            serial,
            x,
            y);
    }

    void zxdg_toplevel_v6::move(struct ::wl_seat *seat, uint32_t serial)
    {
        zxdg_toplevel_v6_move(
            m_zxdg_toplevel_v6,
            seat,
            serial);
    }

    void zxdg_toplevel_v6::resize(struct ::wl_seat *seat, uint32_t serial, uint32_t edges)
    {
        zxdg_toplevel_v6_resize(
            m_zxdg_toplevel_v6,
            seat,
            serial,
            edges);
    }

    void zxdg_toplevel_v6::set_max_size(int32_t width, int32_t height)
    {
        zxdg_toplevel_v6_set_max_size(
            m_zxdg_toplevel_v6,
            width,
            height);
    }

    void zxdg_toplevel_v6::set_min_size(int32_t width, int32_t height)
    {
        zxdg_toplevel_v6_set_min_size(
            m_zxdg_toplevel_v6,
            width,
            height);
    }

    void zxdg_toplevel_v6::set_maximized()
    {
        zxdg_toplevel_v6_set_maximized(
            m_zxdg_toplevel_v6);
    }

    void zxdg_toplevel_v6::unset_maximized()
    {
        zxdg_toplevel_v6_unset_maximized(
            m_zxdg_toplevel_v6);
    }

    void zxdg_toplevel_v6::set_fullscreen(struct ::wl_output *output)
    {
        zxdg_toplevel_v6_set_fullscreen(
            m_zxdg_toplevel_v6,
            output);
    }

    void zxdg_toplevel_v6::unset_fullscreen()
    {
        zxdg_toplevel_v6_unset_fullscreen(
            m_zxdg_toplevel_v6);
    }

    void zxdg_toplevel_v6::set_minimized()
    {
        zxdg_toplevel_v6_set_minimized(
            m_zxdg_toplevel_v6);
    }

    void zxdg_toplevel_v6::zxdg_toplevel_v6_configure(int32_t , int32_t , wl_array *)
    {
    }

    void zxdg_toplevel_v6::handle_configure(
        void *data,
        struct ::zxdg_toplevel_v6 *object,
        int32_t width,
        int32_t height,
        wl_array *states)
    {
        Q_UNUSED(object);
        static_cast<zxdg_toplevel_v6 *>(data)->zxdg_toplevel_v6_configure(
            width,
            height,
            states);
    }

    void zxdg_toplevel_v6::zxdg_toplevel_v6_close()
    {
    }

    void zxdg_toplevel_v6::handle_close(
        void *data,
        struct ::zxdg_toplevel_v6 *object)
    {
        Q_UNUSED(object);
        static_cast<zxdg_toplevel_v6 *>(data)->zxdg_toplevel_v6_close();
    }

    const struct zxdg_toplevel_v6_listener zxdg_toplevel_v6::m_zxdg_toplevel_v6_listener = {
        zxdg_toplevel_v6::handle_configure,
        zxdg_toplevel_v6::handle_close,
    };

    void zxdg_toplevel_v6::init_listener()
    {
        zxdg_toplevel_v6_add_listener(m_zxdg_toplevel_v6, &m_zxdg_toplevel_v6_listener, this);
    }

    zxdg_popup_v6::zxdg_popup_v6(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    zxdg_popup_v6::zxdg_popup_v6(struct ::zxdg_popup_v6 *obj)
        : m_zxdg_popup_v6(obj)
    {
        init_listener();
    }

    zxdg_popup_v6::zxdg_popup_v6()
        : m_zxdg_popup_v6(nullptr)
    {
    }

    zxdg_popup_v6::~zxdg_popup_v6()
    {
    }

    void zxdg_popup_v6::init(struct ::wl_registry *registry, int id, int version)
    {
        m_zxdg_popup_v6 = static_cast<struct ::zxdg_popup_v6 *>(wlRegistryBind(registry, id, &zxdg_popup_v6_interface, version));
        init_listener();
    }

    void zxdg_popup_v6::init(struct ::zxdg_popup_v6 *obj)
    {
        m_zxdg_popup_v6 = obj;
        init_listener();
    }

    zxdg_popup_v6 *zxdg_popup_v6::fromObject(struct ::zxdg_popup_v6 *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_zxdg_popup_v6_listener)
            return nullptr;
        return static_cast<zxdg_popup_v6 *>(zxdg_popup_v6_get_user_data(object));
    }

    bool zxdg_popup_v6::isInitialized() const
    {
        return m_zxdg_popup_v6 != nullptr;
    }

    const struct wl_interface *zxdg_popup_v6::interface()
    {
        return &::zxdg_popup_v6_interface;
    }

    void zxdg_popup_v6::destroy()
    {
        zxdg_popup_v6_destroy(
            m_zxdg_popup_v6);
        m_zxdg_popup_v6 = nullptr;
    }

    void zxdg_popup_v6::grab(struct ::wl_seat *seat, uint32_t serial)
    {
        zxdg_popup_v6_grab(
            m_zxdg_popup_v6,
            seat,
            serial);
    }

    void zxdg_popup_v6::zxdg_popup_v6_configure(int32_t , int32_t , int32_t , int32_t )
    {
    }

    void zxdg_popup_v6::handle_configure(
        void *data,
        struct ::zxdg_popup_v6 *object,
        int32_t x,
        int32_t y,
        int32_t width,
        int32_t height)
    {
        Q_UNUSED(object);
        static_cast<zxdg_popup_v6 *>(data)->zxdg_popup_v6_configure(
            x,
            y,
            width,
            height);
    }

    void zxdg_popup_v6::zxdg_popup_v6_popup_done()
    {
    }

    void zxdg_popup_v6::handle_popup_done(
        void *data,
        struct ::zxdg_popup_v6 *object)
    {
        Q_UNUSED(object);
        static_cast<zxdg_popup_v6 *>(data)->zxdg_popup_v6_popup_done();
    }

    const struct zxdg_popup_v6_listener zxdg_popup_v6::m_zxdg_popup_v6_listener = {
        zxdg_popup_v6::handle_configure,
        zxdg_popup_v6::handle_popup_done,
    };

    void zxdg_popup_v6::init_listener()
    {
        zxdg_popup_v6_add_listener(m_zxdg_popup_v6, &m_zxdg_popup_v6_listener, this);
    }
}

QT_WARNING_POP
QT_END_NAMESPACE
