/****************************************************************************
** Meta object code from reading C++ file 'qwaylandwlshell.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/extensions/qwaylandwlshell.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandwlshell.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandWlShell_t {
    QByteArrayData data[11];
    char stringdata0[166];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandWlShell_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandWlShell_t qt_meta_stringdata_QWaylandWlShell = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QWaylandWlShell"
QT_MOC_LITERAL(1, 16, 23), // "wlShellSurfaceRequested"
QT_MOC_LITERAL(2, 40, 0), // ""
QT_MOC_LITERAL(3, 41, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(4, 58, 7), // "surface"
QT_MOC_LITERAL(5, 66, 16), // "QWaylandResource"
QT_MOC_LITERAL(6, 83, 8), // "resource"
QT_MOC_LITERAL(7, 92, 21), // "wlShellSurfaceCreated"
QT_MOC_LITERAL(8, 114, 23), // "QWaylandWlShellSurface*"
QT_MOC_LITERAL(9, 138, 12), // "shellSurface"
QT_MOC_LITERAL(10, 151, 14) // "closeAllPopups"

    },
    "QWaylandWlShell\0wlShellSurfaceRequested\0"
    "\0QWaylandSurface*\0surface\0QWaylandResource\0"
    "resource\0wlShellSurfaceCreated\0"
    "QWaylandWlShellSurface*\0shellSurface\0"
    "closeAllPopups"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandWlShell[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   29,    2, 0x06 /* Public */,
       7,    1,   34,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,   37,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, 0x80000000 | 8,    9,

 // slots: parameters
    QMetaType::Void,

       0        // eod
};

void QWaylandWlShell::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandWlShell *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->wlShellSurfaceRequested((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< const QWaylandResource(*)>(_a[2]))); break;
        case 1: _t->wlShellSurfaceCreated((*reinterpret_cast< QWaylandWlShellSurface*(*)>(_a[1]))); break;
        case 2: _t->closeAllPopups(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandResource >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandWlShellSurface* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandWlShell::*)(QWaylandSurface * , const QWaylandResource & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShell::wlShellSurfaceRequested)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShell::*)(QWaylandWlShellSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShell::wlShellSurfaceCreated)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QWaylandWlShell::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandShellTemplate<QWaylandWlShell>::staticMetaObject>(),
    qt_meta_stringdata_QWaylandWlShell.data,
    qt_meta_data_QWaylandWlShell,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandWlShell::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandWlShell::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandWlShell.stringdata0))
        return static_cast<void*>(this);
    return QWaylandShellTemplate<QWaylandWlShell>::qt_metacast(_clname);
}

int QWaylandWlShell::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandShellTemplate<QWaylandWlShell>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QWaylandWlShell::wlShellSurfaceRequested(QWaylandSurface * _t1, const QWaylandResource & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QWaylandWlShell::wlShellSurfaceCreated(QWaylandWlShellSurface * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_QWaylandWlShellSurface_t {
    QByteArrayData data[57];
    char stringdata0[685];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandWlShellSurface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandWlShellSurface_t qt_meta_stringdata_QWaylandWlShellSurface = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QWaylandWlShellSurface"
QT_MOC_LITERAL(1, 23, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 39, 4), // "data"
QT_MOC_LITERAL(3, 44, 14), // "surfaceChanged"
QT_MOC_LITERAL(4, 59, 0), // ""
QT_MOC_LITERAL(5, 60, 12), // "shellChanged"
QT_MOC_LITERAL(6, 73, 12), // "titleChanged"
QT_MOC_LITERAL(7, 86, 16), // "classNameChanged"
QT_MOC_LITERAL(8, 103, 4), // "pong"
QT_MOC_LITERAL(9, 108, 9), // "startMove"
QT_MOC_LITERAL(10, 118, 13), // "QWaylandSeat*"
QT_MOC_LITERAL(11, 132, 4), // "seat"
QT_MOC_LITERAL(12, 137, 11), // "startResize"
QT_MOC_LITERAL(13, 149, 10), // "ResizeEdge"
QT_MOC_LITERAL(14, 160, 5), // "edges"
QT_MOC_LITERAL(15, 166, 18), // "setDefaultToplevel"
QT_MOC_LITERAL(16, 185, 12), // "setTransient"
QT_MOC_LITERAL(17, 198, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(18, 215, 13), // "parentSurface"
QT_MOC_LITERAL(19, 229, 16), // "relativeToParent"
QT_MOC_LITERAL(20, 246, 8), // "inactive"
QT_MOC_LITERAL(21, 255, 13), // "setFullScreen"
QT_MOC_LITERAL(22, 269, 16), // "FullScreenMethod"
QT_MOC_LITERAL(23, 286, 6), // "method"
QT_MOC_LITERAL(24, 293, 9), // "framerate"
QT_MOC_LITERAL(25, 303, 15), // "QWaylandOutput*"
QT_MOC_LITERAL(26, 319, 6), // "output"
QT_MOC_LITERAL(27, 326, 8), // "setPopup"
QT_MOC_LITERAL(28, 335, 12), // "setMaximized"
QT_MOC_LITERAL(29, 348, 4), // "ping"
QT_MOC_LITERAL(30, 353, 10), // "initialize"
QT_MOC_LITERAL(31, 364, 16), // "QWaylandWlShell*"
QT_MOC_LITERAL(32, 381, 5), // "shell"
QT_MOC_LITERAL(33, 387, 7), // "surface"
QT_MOC_LITERAL(34, 395, 16), // "QWaylandResource"
QT_MOC_LITERAL(35, 412, 8), // "resource"
QT_MOC_LITERAL(36, 421, 13), // "sizeForResize"
QT_MOC_LITERAL(37, 435, 4), // "size"
QT_MOC_LITERAL(38, 440, 5), // "delta"
QT_MOC_LITERAL(39, 446, 13), // "sendConfigure"
QT_MOC_LITERAL(40, 460, 13), // "sendPopupDone"
QT_MOC_LITERAL(41, 474, 25), // "QQmlListProperty<QObject>"
QT_MOC_LITERAL(42, 500, 5), // "title"
QT_MOC_LITERAL(43, 506, 9), // "className"
QT_MOC_LITERAL(44, 516, 17), // "DefaultFullScreen"
QT_MOC_LITERAL(45, 534, 15), // "ScaleFullScreen"
QT_MOC_LITERAL(46, 550, 16), // "DriverFullScreen"
QT_MOC_LITERAL(47, 567, 14), // "FillFullScreen"
QT_MOC_LITERAL(48, 582, 8), // "NoneEdge"
QT_MOC_LITERAL(49, 591, 7), // "TopEdge"
QT_MOC_LITERAL(50, 599, 10), // "BottomEdge"
QT_MOC_LITERAL(51, 610, 8), // "LeftEdge"
QT_MOC_LITERAL(52, 619, 11), // "TopLeftEdge"
QT_MOC_LITERAL(53, 631, 14), // "BottomLeftEdge"
QT_MOC_LITERAL(54, 646, 9), // "RightEdge"
QT_MOC_LITERAL(55, 656, 12), // "TopRightEdge"
QT_MOC_LITERAL(56, 669, 15) // "BottomRightEdge"

    },
    "QWaylandWlShellSurface\0DefaultProperty\0"
    "data\0surfaceChanged\0\0shellChanged\0"
    "titleChanged\0classNameChanged\0pong\0"
    "startMove\0QWaylandSeat*\0seat\0startResize\0"
    "ResizeEdge\0edges\0setDefaultToplevel\0"
    "setTransient\0QWaylandSurface*\0"
    "parentSurface\0relativeToParent\0inactive\0"
    "setFullScreen\0FullScreenMethod\0method\0"
    "framerate\0QWaylandOutput*\0output\0"
    "setPopup\0setMaximized\0ping\0initialize\0"
    "QWaylandWlShell*\0shell\0surface\0"
    "QWaylandResource\0resource\0sizeForResize\0"
    "size\0delta\0sendConfigure\0sendPopupDone\0"
    "QQmlListProperty<QObject>\0title\0"
    "className\0DefaultFullScreen\0ScaleFullScreen\0"
    "DriverFullScreen\0FillFullScreen\0"
    "NoneEdge\0TopEdge\0BottomEdge\0LeftEdge\0"
    "TopLeftEdge\0BottomLeftEdge\0RightEdge\0"
    "TopRightEdge\0BottomRightEdge"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandWlShellSurface[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
      17,   16, // methods
       5,  160, // properties
       2,  180, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,  101,    4, 0x06 /* Public */,
       5,    0,  102,    4, 0x06 /* Public */,
       6,    0,  103,    4, 0x06 /* Public */,
       7,    0,  104,    4, 0x06 /* Public */,
       8,    0,  105,    4, 0x06 /* Public */,
       9,    1,  106,    4, 0x06 /* Public */,
      12,    2,  109,    4, 0x06 /* Public */,
      15,    0,  114,    4, 0x06 /* Public */,
      16,    3,  115,    4, 0x06 /* Public */,
      21,    3,  122,    4, 0x06 /* Public */,
      27,    3,  129,    4, 0x06 /* Public */,
      28,    1,  136,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      29,    0,  139,    4, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      30,    3,  140,    4, 0x02 /* Public */,
      36,    3,  147,    4, 0x02 /* Public */,
      39,    2,  154,    4, 0x02 /* Public */,
      40,    0,  159,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 13,   11,   14,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17, QMetaType::QPoint, QMetaType::Bool,   18,   19,   20,
    QMetaType::Void, 0x80000000 | 22, QMetaType::UInt, 0x80000000 | 25,   23,   24,   26,
    QMetaType::Void, 0x80000000 | 10, 0x80000000 | 17, QMetaType::QPoint,   11,   18,   19,
    QMetaType::Void, 0x80000000 | 25,   26,

 // slots: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 31, 0x80000000 | 17, 0x80000000 | 34,   32,   33,   35,
    QMetaType::QSize, QMetaType::QSizeF, QMetaType::QPointF, 0x80000000 | 13,   37,   38,   14,
    QMetaType::Void, QMetaType::QSize, 0x80000000 | 13,   37,   14,
    QMetaType::Void,

 // properties: name, type, flags
       2, 0x80000000 | 41, 0x00094009,
      33, 0x80000000 | 17, 0x00495009,
      32, 0x80000000 | 31, 0x00495009,
      42, QMetaType::QString, 0x00495001,
      43, QMetaType::QString, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
       1,
       2,
       3,

 // enums: name, alias, flags, count, data
      22,   22, 0x0,    4,  190,
      13,   13, 0x0,    9,  198,

 // enum data: key, value
      44, uint(QWaylandWlShellSurface::DefaultFullScreen),
      45, uint(QWaylandWlShellSurface::ScaleFullScreen),
      46, uint(QWaylandWlShellSurface::DriverFullScreen),
      47, uint(QWaylandWlShellSurface::FillFullScreen),
      48, uint(QWaylandWlShellSurface::NoneEdge),
      49, uint(QWaylandWlShellSurface::TopEdge),
      50, uint(QWaylandWlShellSurface::BottomEdge),
      51, uint(QWaylandWlShellSurface::LeftEdge),
      52, uint(QWaylandWlShellSurface::TopLeftEdge),
      53, uint(QWaylandWlShellSurface::BottomLeftEdge),
      54, uint(QWaylandWlShellSurface::RightEdge),
      55, uint(QWaylandWlShellSurface::TopRightEdge),
      56, uint(QWaylandWlShellSurface::BottomRightEdge),

       0        // eod
};

void QWaylandWlShellSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandWlShellSurface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->surfaceChanged(); break;
        case 1: _t->shellChanged(); break;
        case 2: _t->titleChanged(); break;
        case 3: _t->classNameChanged(); break;
        case 4: _t->pong(); break;
        case 5: _t->startMove((*reinterpret_cast< QWaylandSeat*(*)>(_a[1]))); break;
        case 6: _t->startResize((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< ResizeEdge(*)>(_a[2]))); break;
        case 7: _t->setDefaultToplevel(); break;
        case 8: _t->setTransient((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 9: _t->setFullScreen((*reinterpret_cast< FullScreenMethod(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< QWaylandOutput*(*)>(_a[3]))); break;
        case 10: _t->setPopup((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2])),(*reinterpret_cast< const QPoint(*)>(_a[3]))); break;
        case 11: _t->setMaximized((*reinterpret_cast< QWaylandOutput*(*)>(_a[1]))); break;
        case 12: _t->ping(); break;
        case 13: _t->initialize((*reinterpret_cast< QWaylandWlShell*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2])),(*reinterpret_cast< const QWaylandResource(*)>(_a[3]))); break;
        case 14: { QSize _r = _t->sizeForResize((*reinterpret_cast< const QSizeF(*)>(_a[1])),(*reinterpret_cast< const QPointF(*)>(_a[2])),(*reinterpret_cast< ResizeEdge(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QSize*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->sendConfigure((*reinterpret_cast< const QSize(*)>(_a[1])),(*reinterpret_cast< ResizeEdge(*)>(_a[2]))); break;
        case 16: _t->sendPopupDone(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandResource >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandWlShell* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandWlShellSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::surfaceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::shellChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::titleChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::classNameChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::pong)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)(QWaylandSeat * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::startMove)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)(QWaylandSeat * , ResizeEdge );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::startResize)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::setDefaultToplevel)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)(QWaylandSurface * , const QPoint & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::setTransient)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)(FullScreenMethod , uint , QWaylandOutput * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::setFullScreen)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)(QWaylandSeat * , QWaylandSurface * , const QPoint & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::setPopup)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QWaylandWlShellSurface::*)(QWaylandOutput * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandWlShellSurface::setMaximized)) {
                *result = 11;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandWlShell* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandWlShellSurface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->data(); break;
        case 1: *reinterpret_cast< QWaylandSurface**>(_v) = _t->surface(); break;
        case 2: *reinterpret_cast< QWaylandWlShell**>(_v) = _t->shell(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->title(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->className(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandWlShellSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandShellSurfaceTemplate<QWaylandWlShellSurface>::staticMetaObject>(),
    qt_meta_stringdata_QWaylandWlShellSurface.data,
    qt_meta_data_QWaylandWlShellSurface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandWlShellSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandWlShellSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandWlShellSurface.stringdata0))
        return static_cast<void*>(this);
    return QWaylandShellSurfaceTemplate<QWaylandWlShellSurface>::qt_metacast(_clname);
}

int QWaylandWlShellSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandShellSurfaceTemplate<QWaylandWlShellSurface>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandWlShellSurface::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandWlShellSurface::shellChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWaylandWlShellSurface::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QWaylandWlShellSurface::classNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QWaylandWlShellSurface::pong()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QWaylandWlShellSurface::startMove(QWaylandSeat * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QWaylandWlShellSurface::startResize(QWaylandSeat * _t1, ResizeEdge _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QWaylandWlShellSurface::setDefaultToplevel()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QWaylandWlShellSurface::setTransient(QWaylandSurface * _t1, const QPoint & _t2, bool _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QWaylandWlShellSurface::setFullScreen(FullScreenMethod _t1, uint _t2, QWaylandOutput * _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QWaylandWlShellSurface::setPopup(QWaylandSeat * _t1, QWaylandSurface * _t2, const QPoint & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QWaylandWlShellSurface::setMaximized(QWaylandOutput * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
