/****************************************************************************
** Meta object code from reading C++ file 'qwaylandcompositor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/compositor_api/qwaylandcompositor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandcompositor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandCompositor_t {
    QByteArrayData data[41];
    char stringdata0[591];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandCompositor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandCompositor_t qt_meta_stringdata_QWaylandCompositor = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QWaylandCompositor"
QT_MOC_LITERAL(1, 19, 14), // "createdChanged"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 17), // "socketNameChanged"
QT_MOC_LITERAL(4, 53, 10), // "socketName"
QT_MOC_LITERAL(5, 64, 24), // "retainedSelectionChanged"
QT_MOC_LITERAL(6, 89, 17), // "retainedSelection"
QT_MOC_LITERAL(7, 107, 16), // "surfaceRequested"
QT_MOC_LITERAL(8, 124, 15), // "QWaylandClient*"
QT_MOC_LITERAL(9, 140, 6), // "client"
QT_MOC_LITERAL(10, 147, 2), // "id"
QT_MOC_LITERAL(11, 150, 7), // "version"
QT_MOC_LITERAL(12, 158, 14), // "surfaceCreated"
QT_MOC_LITERAL(13, 173, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(14, 190, 7), // "surface"
QT_MOC_LITERAL(15, 198, 25), // "surfaceAboutToBeDestroyed"
QT_MOC_LITERAL(16, 224, 17), // "subsurfaceChanged"
QT_MOC_LITERAL(17, 242, 5), // "child"
QT_MOC_LITERAL(18, 248, 6), // "parent"
QT_MOC_LITERAL(19, 255, 20), // "defaultOutputChanged"
QT_MOC_LITERAL(20, 276, 18), // "defaultSeatChanged"
QT_MOC_LITERAL(21, 295, 13), // "QWaylandSeat*"
QT_MOC_LITERAL(22, 309, 9), // "newDevice"
QT_MOC_LITERAL(23, 319, 9), // "oldDevice"
QT_MOC_LITERAL(24, 329, 38), // "useHardwareIntegrationExtensi..."
QT_MOC_LITERAL(25, 368, 11), // "outputAdded"
QT_MOC_LITERAL(26, 380, 15), // "QWaylandOutput*"
QT_MOC_LITERAL(27, 396, 6), // "output"
QT_MOC_LITERAL(28, 403, 13), // "outputRemoved"
QT_MOC_LITERAL(29, 417, 20), // "processWaylandEvents"
QT_MOC_LITERAL(30, 438, 19), // "addSocketDescriptor"
QT_MOC_LITERAL(31, 458, 2), // "fd"
QT_MOC_LITERAL(32, 461, 23), // "destroyClientForSurface"
QT_MOC_LITERAL(33, 485, 13), // "destroyClient"
QT_MOC_LITERAL(34, 499, 9), // "outputFor"
QT_MOC_LITERAL(35, 509, 8), // "QWindow*"
QT_MOC_LITERAL(36, 518, 6), // "window"
QT_MOC_LITERAL(37, 525, 7), // "created"
QT_MOC_LITERAL(38, 533, 13), // "defaultOutput"
QT_MOC_LITERAL(39, 547, 31), // "useHardwareIntegrationExtension"
QT_MOC_LITERAL(40, 579, 11) // "defaultSeat"

    },
    "QWaylandCompositor\0createdChanged\0\0"
    "socketNameChanged\0socketName\0"
    "retainedSelectionChanged\0retainedSelection\0"
    "surfaceRequested\0QWaylandClient*\0"
    "client\0id\0version\0surfaceCreated\0"
    "QWaylandSurface*\0surface\0"
    "surfaceAboutToBeDestroyed\0subsurfaceChanged\0"
    "child\0parent\0defaultOutputChanged\0"
    "defaultSeatChanged\0QWaylandSeat*\0"
    "newDevice\0oldDevice\0"
    "useHardwareIntegrationExtensionChanged\0"
    "outputAdded\0QWaylandOutput*\0output\0"
    "outputRemoved\0processWaylandEvents\0"
    "addSocketDescriptor\0fd\0destroyClientForSurface\0"
    "destroyClient\0outputFor\0QWindow*\0"
    "window\0created\0defaultOutput\0"
    "useHardwareIntegrationExtension\0"
    "defaultSeat"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandCompositor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       6,  150, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      12,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   99,    2, 0x06 /* Public */,
       3,    1,  100,    2, 0x06 /* Public */,
       5,    1,  103,    2, 0x06 /* Public */,
       7,    3,  106,    2, 0x06 /* Public */,
      12,    1,  113,    2, 0x06 /* Public */,
      15,    1,  116,    2, 0x06 /* Public */,
      16,    2,  119,    2, 0x06 /* Public */,
      19,    0,  124,    2, 0x06 /* Public */,
      20,    2,  125,    2, 0x06 /* Public */,
      24,    0,  130,    2, 0x06 /* Public */,
      25,    1,  131,    2, 0x06 /* Public */,
      28,    1,  134,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      29,    0,  137,    2, 0x0a /* Public */,

 // methods: name, argc, parameters, tag, flags
      30,    1,  138,    2, 0x02 /* Public */,
      32,    1,  141,    2, 0x02 /* Public */,
      33,    1,  144,    2, 0x02 /* Public */,
      34,    1,  147,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QByteArray,    4,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void, 0x80000000 | 8, QMetaType::UInt, QMetaType::Int,    9,   10,   11,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 13, 0x80000000 | 13,   17,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 21, 0x80000000 | 21,   22,   23,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 26,   27,
    QMetaType::Void, 0x80000000 | 26,   27,

 // slots: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, QMetaType::Int,   31,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 8,    9,
    0x80000000 | 26, 0x80000000 | 35,   36,

 // properties: name, type, flags
       4, QMetaType::QByteArray, 0x00495103,
      37, QMetaType::Bool, 0x00495001,
       6, QMetaType::Bool, 0x00495003,
      38, 0x80000000 | 26, 0x0049510b,
      39, QMetaType::Bool, 0x00495103,
      40, 0x80000000 | 21, 0x00495009,

 // properties: notify_signal_id
       1,
       0,
       2,
       7,
       9,
       8,

       0        // eod
};

void QWaylandCompositor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandCompositor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->createdChanged(); break;
        case 1: _t->socketNameChanged((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 2: _t->retainedSelectionChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->surfaceRequested((*reinterpret_cast< QWaylandClient*(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 4: _t->surfaceCreated((*reinterpret_cast< QWaylandSurface*(*)>(_a[1]))); break;
        case 5: _t->surfaceAboutToBeDestroyed((*reinterpret_cast< QWaylandSurface*(*)>(_a[1]))); break;
        case 6: _t->subsurfaceChanged((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2]))); break;
        case 7: _t->defaultOutputChanged(); break;
        case 8: _t->defaultSeatChanged((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< QWaylandSeat*(*)>(_a[2]))); break;
        case 9: _t->useHardwareIntegrationExtensionChanged(); break;
        case 10: _t->outputAdded((*reinterpret_cast< QWaylandOutput*(*)>(_a[1]))); break;
        case 11: _t->outputRemoved((*reinterpret_cast< QWaylandOutput*(*)>(_a[1]))); break;
        case 12: _t->processWaylandEvents(); break;
        case 13: _t->addSocketDescriptor((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->destroyClientForSurface((*reinterpret_cast< QWaylandSurface*(*)>(_a[1]))); break;
        case 15: _t->destroyClient((*reinterpret_cast< QWaylandClient*(*)>(_a[1]))); break;
        case 16: { QWaylandOutput* _r = _t->outputFor((*reinterpret_cast< QWindow*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QWaylandOutput**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandOutput* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandOutput* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandCompositor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::createdChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)(const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::socketNameChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::retainedSelectionChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)(QWaylandClient * , uint , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::surfaceRequested)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)(QWaylandSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::surfaceCreated)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)(QWaylandSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::surfaceAboutToBeDestroyed)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)(QWaylandSurface * , QWaylandSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::subsurfaceChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::defaultOutputChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)(QWaylandSeat * , QWaylandSeat * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::defaultSeatChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::useHardwareIntegrationExtensionChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)(QWaylandOutput * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::outputAdded)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QWaylandCompositor::*)(QWaylandOutput * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandCompositor::outputRemoved)) {
                *result = 11;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandOutput* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandCompositor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QByteArray*>(_v) = _t->socketName(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->isCreated(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->retainedSelectionEnabled(); break;
        case 3: *reinterpret_cast< QWaylandOutput**>(_v) = _t->defaultOutput(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->useHardwareIntegrationExtension(); break;
        case 5: *reinterpret_cast< QWaylandSeat**>(_v) = _t->defaultSeat(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QWaylandCompositor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSocketName(*reinterpret_cast< QByteArray*>(_v)); break;
        case 2: _t->setRetainedSelectionEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 3: _t->setDefaultOutput(*reinterpret_cast< QWaylandOutput**>(_v)); break;
        case 4: _t->setUseHardwareIntegrationExtension(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandCompositor::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandObject::staticMetaObject>(),
    qt_meta_stringdata_QWaylandCompositor.data,
    qt_meta_data_QWaylandCompositor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandCompositor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandCompositor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandCompositor.stringdata0))
        return static_cast<void*>(this);
    return QWaylandObject::qt_metacast(_clname);
}

int QWaylandCompositor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandCompositor::createdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandCompositor::socketNameChanged(const QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QWaylandCompositor::retainedSelectionChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QWaylandCompositor::surfaceRequested(QWaylandClient * _t1, uint _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QWaylandCompositor::surfaceCreated(QWaylandSurface * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QWaylandCompositor::surfaceAboutToBeDestroyed(QWaylandSurface * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QWaylandCompositor::subsurfaceChanged(QWaylandSurface * _t1, QWaylandSurface * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QWaylandCompositor::defaultOutputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QWaylandCompositor::defaultSeatChanged(QWaylandSeat * _t1, QWaylandSeat * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QWaylandCompositor::useHardwareIntegrationExtensionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QWaylandCompositor::outputAdded(QWaylandOutput * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QWaylandCompositor::outputRemoved(QWaylandOutput * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
