// This file was generated by qtwaylandscanner
// source file is /scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VckScFull/build/tmp/work/vck_sc_zynqmp-xilinx-linux/qtwayland/5.15.4+gitAUTOINC+f1e6c8764d-r0/git/src/extensions/qt-texture-sharing-unstable-v1.xml

#ifndef QT_WAYLAND_QT_TEXTURE_SHARING_UNSTABLE_V1
#define QT_WAYLAND_QT_TEXTURE_SHARING_UNSTABLE_V1

#include "wayland-qt-texture-sharing-unstable-v1-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")

namespace QtWayland {
    class  zqt_texture_sharing_v1
    {
    public:
        zqt_texture_sharing_v1(struct ::wl_registry *registry, int id, int version);
        zqt_texture_sharing_v1(struct ::zqt_texture_sharing_v1 *object);
        zqt_texture_sharing_v1();

        virtual ~zqt_texture_sharing_v1();

        void init(struct ::wl_registry *registry, int id, int version);
        void init(struct ::zqt_texture_sharing_v1 *object);

        struct ::zqt_texture_sharing_v1 *object() { return m_zqt_texture_sharing_v1; }
        const struct ::zqt_texture_sharing_v1 *object() const { return m_zqt_texture_sharing_v1; }
        static zqt_texture_sharing_v1 *fromObject(struct ::zqt_texture_sharing_v1 *object);

        bool isInitialized() const;

        static const struct ::wl_interface *interface();

        void request_image(const QString &key);
        void abandon_image(const QString &key);

    protected:
        virtual void zqt_texture_sharing_v1_image_failed(const QString &key, const QString &error_message);
        virtual void zqt_texture_sharing_v1_provide_buffer(struct ::qt_server_buffer *buffer, const QString &key);

    private:
        void init_listener();
        static const struct zqt_texture_sharing_v1_listener m_zqt_texture_sharing_v1_listener;
        static void handle_image_failed(
            void *data,
            struct ::zqt_texture_sharing_v1 *object,
            const char *key,
            const char *error_message);
        static void handle_provide_buffer(
            void *data,
            struct ::zqt_texture_sharing_v1 *object,
            struct ::qt_server_buffer *buffer,
            const char *key);
        struct ::zqt_texture_sharing_v1 *m_zqt_texture_sharing_v1;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
