// This file was generated by qtwaylandscanner
// source file is /scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VckScFull/build/tmp/work/vck_sc_zynqmp-xilinx-linux/qtwayland/5.15.4+gitAUTOINC+f1e6c8764d-r0/git/src/extensions/shm-emulation-server-buffer.xml

#ifndef QT_WAYLAND_SHM_EMULATION_SERVER_BUFFER
#define QT_WAYLAND_SHM_EMULATION_SERVER_BUFFER

#include "wayland-shm-emulation-server-buffer-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")

namespace QtWayland {
    class  qt_shm_emulation_server_buffer
    {
    public:
        qt_shm_emulation_server_buffer(struct ::wl_registry *registry, int id, int version);
        qt_shm_emulation_server_buffer(struct ::qt_shm_emulation_server_buffer *object);
        qt_shm_emulation_server_buffer();

        virtual ~qt_shm_emulation_server_buffer();

        void init(struct ::wl_registry *registry, int id, int version);
        void init(struct ::qt_shm_emulation_server_buffer *object);

        struct ::qt_shm_emulation_server_buffer *object() { return m_qt_shm_emulation_server_buffer; }
        const struct ::qt_shm_emulation_server_buffer *object() const { return m_qt_shm_emulation_server_buffer; }
        static qt_shm_emulation_server_buffer *fromObject(struct ::qt_shm_emulation_server_buffer *object);

        bool isInitialized() const;

        static const struct ::wl_interface *interface();

        enum format {
            format_RGBA32 = 0,
            format_A8 = 1,
        };

    protected:
        virtual void shm_emulation_server_buffer_server_buffer_created(struct ::qt_server_buffer *id, const QString &key, int32_t width, int32_t height, int32_t bytes_per_line, int32_t format);

    private:
        void init_listener();
        static const struct qt_shm_emulation_server_buffer_listener m_qt_shm_emulation_server_buffer_listener;
        static void handle_server_buffer_created(
            void *data,
            struct ::qt_shm_emulation_server_buffer *object,
            struct ::qt_server_buffer *id,
            const char *key,
            int32_t width,
            int32_t height,
            int32_t bytes_per_line,
            int32_t format);
        struct ::qt_shm_emulation_server_buffer *m_qt_shm_emulation_server_buffer;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
