/****************************************************************************
** Meta object code from reading C++ file 'qwaylandoutput.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/compositor_api/qwaylandoutput.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandoutput.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandOutput_t {
    QByteArrayData data[52];
    char stringdata0[850];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandOutput_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandOutput_t qt_meta_stringdata_QWaylandOutput = {
    {
QT_MOC_LITERAL(0, 0, 14), // "QWaylandOutput"
QT_MOC_LITERAL(1, 15, 17), // "compositorChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 13), // "windowChanged"
QT_MOC_LITERAL(4, 48, 15), // "positionChanged"
QT_MOC_LITERAL(5, 64, 15), // "geometryChanged"
QT_MOC_LITERAL(6, 80, 9), // "modeAdded"
QT_MOC_LITERAL(7, 90, 18), // "currentModeChanged"
QT_MOC_LITERAL(8, 109, 24), // "availableGeometryChanged"
QT_MOC_LITERAL(9, 134, 19), // "physicalSizeChanged"
QT_MOC_LITERAL(10, 154, 18), // "scaleFactorChanged"
QT_MOC_LITERAL(11, 173, 15), // "subpixelChanged"
QT_MOC_LITERAL(12, 189, 16), // "transformChanged"
QT_MOC_LITERAL(13, 206, 24), // "sizeFollowsWindowChanged"
QT_MOC_LITERAL(14, 231, 30), // "physicalSizeFollowsSizeChanged"
QT_MOC_LITERAL(15, 262, 19), // "manufacturerChanged"
QT_MOC_LITERAL(16, 282, 12), // "modelChanged"
QT_MOC_LITERAL(17, 295, 15), // "windowDestroyed"
QT_MOC_LITERAL(18, 311, 36), // "_q_handleMaybeWindowPixelSize..."
QT_MOC_LITERAL(19, 348, 24), // "_q_handleWindowDestroyed"
QT_MOC_LITERAL(20, 373, 10), // "compositor"
QT_MOC_LITERAL(21, 384, 19), // "QWaylandCompositor*"
QT_MOC_LITERAL(22, 404, 6), // "window"
QT_MOC_LITERAL(23, 411, 8), // "QWindow*"
QT_MOC_LITERAL(24, 420, 12), // "manufacturer"
QT_MOC_LITERAL(25, 433, 5), // "model"
QT_MOC_LITERAL(26, 439, 8), // "position"
QT_MOC_LITERAL(27, 448, 8), // "geometry"
QT_MOC_LITERAL(28, 457, 17), // "availableGeometry"
QT_MOC_LITERAL(29, 475, 12), // "physicalSize"
QT_MOC_LITERAL(30, 488, 8), // "subpixel"
QT_MOC_LITERAL(31, 497, 24), // "QWaylandOutput::Subpixel"
QT_MOC_LITERAL(32, 522, 9), // "transform"
QT_MOC_LITERAL(33, 532, 25), // "QWaylandOutput::Transform"
QT_MOC_LITERAL(34, 558, 11), // "scaleFactor"
QT_MOC_LITERAL(35, 570, 17), // "sizeFollowsWindow"
QT_MOC_LITERAL(36, 588, 8), // "Subpixel"
QT_MOC_LITERAL(37, 597, 15), // "SubpixelUnknown"
QT_MOC_LITERAL(38, 613, 12), // "SubpixelNone"
QT_MOC_LITERAL(39, 626, 21), // "SubpixelHorizontalRgb"
QT_MOC_LITERAL(40, 648, 21), // "SubpixelHorizontalBgr"
QT_MOC_LITERAL(41, 670, 19), // "SubpixelVerticalRgb"
QT_MOC_LITERAL(42, 690, 19), // "SubpixelVerticalBgr"
QT_MOC_LITERAL(43, 710, 9), // "Transform"
QT_MOC_LITERAL(44, 720, 15), // "TransformNormal"
QT_MOC_LITERAL(45, 736, 11), // "Transform90"
QT_MOC_LITERAL(46, 748, 12), // "Transform180"
QT_MOC_LITERAL(47, 761, 12), // "Transform270"
QT_MOC_LITERAL(48, 774, 16), // "TransformFlipped"
QT_MOC_LITERAL(49, 791, 18), // "TransformFlipped90"
QT_MOC_LITERAL(50, 810, 19), // "TransformFlipped180"
QT_MOC_LITERAL(51, 830, 19) // "TransformFlipped270"

    },
    "QWaylandOutput\0compositorChanged\0\0"
    "windowChanged\0positionChanged\0"
    "geometryChanged\0modeAdded\0currentModeChanged\0"
    "availableGeometryChanged\0physicalSizeChanged\0"
    "scaleFactorChanged\0subpixelChanged\0"
    "transformChanged\0sizeFollowsWindowChanged\0"
    "physicalSizeFollowsSizeChanged\0"
    "manufacturerChanged\0modelChanged\0"
    "windowDestroyed\0_q_handleMaybeWindowPixelSizeChanged\0"
    "_q_handleWindowDestroyed\0compositor\0"
    "QWaylandCompositor*\0window\0QWindow*\0"
    "manufacturer\0model\0position\0geometry\0"
    "availableGeometry\0physicalSize\0subpixel\0"
    "QWaylandOutput::Subpixel\0transform\0"
    "QWaylandOutput::Transform\0scaleFactor\0"
    "sizeFollowsWindow\0Subpixel\0SubpixelUnknown\0"
    "SubpixelNone\0SubpixelHorizontalRgb\0"
    "SubpixelHorizontalBgr\0SubpixelVerticalRgb\0"
    "SubpixelVerticalBgr\0Transform\0"
    "TransformNormal\0Transform90\0Transform180\0"
    "Transform270\0TransformFlipped\0"
    "TransformFlipped90\0TransformFlipped180\0"
    "TransformFlipped270"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandOutput[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
      12,  122, // properties
       2,  170, // enums/sets
       0,    0, // constructors
       0,       // flags
      16,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  104,    2, 0x06 /* Public */,
       3,    0,  105,    2, 0x06 /* Public */,
       4,    0,  106,    2, 0x06 /* Public */,
       5,    0,  107,    2, 0x06 /* Public */,
       6,    0,  108,    2, 0x06 /* Public */,
       7,    0,  109,    2, 0x06 /* Public */,
       8,    0,  110,    2, 0x06 /* Public */,
       9,    0,  111,    2, 0x06 /* Public */,
      10,    0,  112,    2, 0x06 /* Public */,
      11,    0,  113,    2, 0x06 /* Public */,
      12,    0,  114,    2, 0x06 /* Public */,
      13,    0,  115,    2, 0x06 /* Public */,
      14,    0,  116,    2, 0x06 /* Public */,
      15,    0,  117,    2, 0x06 /* Public */,
      16,    0,  118,    2, 0x06 /* Public */,
      17,    0,  119,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      18,    0,  120,    2, 0x08 /* Private */,
      19,    0,  121,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      20, 0x80000000 | 21, 0x0049510b,
      22, 0x80000000 | 23, 0x0049510b,
      24, QMetaType::QString, 0x00495103,
      25, QMetaType::QString, 0x00495103,
      26, QMetaType::QPoint, 0x00495103,
      27, QMetaType::QRect, 0x00495001,
      28, QMetaType::QRect, 0x00495103,
      29, QMetaType::QSize, 0x00495103,
      30, 0x80000000 | 31, 0x0049510b,
      32, 0x80000000 | 33, 0x0049510b,
      34, QMetaType::Int, 0x00495103,
      35, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
      13,
      14,
       2,
       3,
       6,
       7,
       9,
      10,
       8,
      11,

 // enums: name, alias, flags, count, data
      36,   36, 0x0,    6,  180,
      43,   43, 0x0,    8,  192,

 // enum data: key, value
      37, uint(QWaylandOutput::SubpixelUnknown),
      38, uint(QWaylandOutput::SubpixelNone),
      39, uint(QWaylandOutput::SubpixelHorizontalRgb),
      40, uint(QWaylandOutput::SubpixelHorizontalBgr),
      41, uint(QWaylandOutput::SubpixelVerticalRgb),
      42, uint(QWaylandOutput::SubpixelVerticalBgr),
      44, uint(QWaylandOutput::TransformNormal),
      45, uint(QWaylandOutput::Transform90),
      46, uint(QWaylandOutput::Transform180),
      47, uint(QWaylandOutput::Transform270),
      48, uint(QWaylandOutput::TransformFlipped),
      49, uint(QWaylandOutput::TransformFlipped90),
      50, uint(QWaylandOutput::TransformFlipped180),
      51, uint(QWaylandOutput::TransformFlipped270),

       0        // eod
};

void QWaylandOutput::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandOutput *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->compositorChanged(); break;
        case 1: _t->windowChanged(); break;
        case 2: _t->positionChanged(); break;
        case 3: _t->geometryChanged(); break;
        case 4: _t->modeAdded(); break;
        case 5: _t->currentModeChanged(); break;
        case 6: _t->availableGeometryChanged(); break;
        case 7: _t->physicalSizeChanged(); break;
        case 8: _t->scaleFactorChanged(); break;
        case 9: _t->subpixelChanged(); break;
        case 10: _t->transformChanged(); break;
        case 11: _t->sizeFollowsWindowChanged(); break;
        case 12: _t->physicalSizeFollowsSizeChanged(); break;
        case 13: _t->manufacturerChanged(); break;
        case 14: _t->modelChanged(); break;
        case 15: _t->windowDestroyed(); break;
        case 16: _t->d_func()->_q_handleMaybeWindowPixelSizeChanged(); break;
        case 17: _t->d_func()->_q_handleWindowDestroyed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::compositorChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::windowChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::positionChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::geometryChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::modeAdded)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::currentModeChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::availableGeometryChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::physicalSizeChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::scaleFactorChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::subpixelChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::transformChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::sizeFollowsWindowChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::physicalSizeFollowsSizeChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::manufacturerChanged)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::modelChanged)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QWaylandOutput::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandOutput::windowDestroyed)) {
                *result = 15;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandOutput *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QWaylandCompositor**>(_v) = _t->compositor(); break;
        case 1: *reinterpret_cast< QWindow**>(_v) = _t->window(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->manufacturer(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->model(); break;
        case 4: *reinterpret_cast< QPoint*>(_v) = _t->position(); break;
        case 5: *reinterpret_cast< QRect*>(_v) = _t->geometry(); break;
        case 6: *reinterpret_cast< QRect*>(_v) = _t->availableGeometry(); break;
        case 7: *reinterpret_cast< QSize*>(_v) = _t->physicalSize(); break;
        case 8: *reinterpret_cast< QWaylandOutput::Subpixel*>(_v) = _t->subpixel(); break;
        case 9: *reinterpret_cast< QWaylandOutput::Transform*>(_v) = _t->transform(); break;
        case 10: *reinterpret_cast< int*>(_v) = _t->scaleFactor(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->sizeFollowsWindow(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QWaylandOutput *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCompositor(*reinterpret_cast< QWaylandCompositor**>(_v)); break;
        case 1: _t->setWindow(*reinterpret_cast< QWindow**>(_v)); break;
        case 2: _t->setManufacturer(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setModel(*reinterpret_cast< QString*>(_v)); break;
        case 4: _t->setPosition(*reinterpret_cast< QPoint*>(_v)); break;
        case 6: _t->setAvailableGeometry(*reinterpret_cast< QRect*>(_v)); break;
        case 7: _t->setPhysicalSize(*reinterpret_cast< QSize*>(_v)); break;
        case 8: _t->setSubpixel(*reinterpret_cast< QWaylandOutput::Subpixel*>(_v)); break;
        case 9: _t->setTransform(*reinterpret_cast< QWaylandOutput::Transform*>(_v)); break;
        case 10: _t->setScaleFactor(*reinterpret_cast< int*>(_v)); break;
        case 11: _t->setSizeFollowsWindow(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QWaylandOutput::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandObject::staticMetaObject>(),
    qt_meta_stringdata_QWaylandOutput.data,
    qt_meta_data_QWaylandOutput,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandOutput::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandOutput::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandOutput.stringdata0))
        return static_cast<void*>(this);
    return QWaylandObject::qt_metacast(_clname);
}

int QWaylandOutput::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 12;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandOutput::compositorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandOutput::windowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWaylandOutput::positionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QWaylandOutput::geometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QWaylandOutput::modeAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QWaylandOutput::currentModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QWaylandOutput::availableGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QWaylandOutput::physicalSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QWaylandOutput::scaleFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QWaylandOutput::subpixelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QWaylandOutput::transformChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QWaylandOutput::sizeFollowsWindowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QWaylandOutput::physicalSizeFollowsSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QWaylandOutput::manufacturerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QWaylandOutput::modelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QWaylandOutput::windowDestroyed()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
