// This file was generated by qtwaylandscanner
// source file is /scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_VckScFull/build/tmp/work/vck_sc_zynqmp-xilinx-linux/qtwayland/5.15.4+gitAUTOINC+f1e6c8764d-r0/git/src/3rdparty/protocol/fullscreen-shell-unstable-v1.xml

#ifndef QT_WAYLAND_FULLSCREEN_SHELL_UNSTABLE_V1
#define QT_WAYLAND_FULLSCREEN_SHELL_UNSTABLE_V1

#include "wayland-fullscreen-shell-unstable-v1-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")

namespace QtWayland {
    class  zwp_fullscreen_shell_v1
    {
    public:
        zwp_fullscreen_shell_v1(struct ::wl_registry *registry, int id, int version);
        zwp_fullscreen_shell_v1(struct ::zwp_fullscreen_shell_v1 *object);
        zwp_fullscreen_shell_v1();

        virtual ~zwp_fullscreen_shell_v1();

        void init(struct ::wl_registry *registry, int id, int version);
        void init(struct ::zwp_fullscreen_shell_v1 *object);

        struct ::zwp_fullscreen_shell_v1 *object() { return m_zwp_fullscreen_shell_v1; }
        const struct ::zwp_fullscreen_shell_v1 *object() const { return m_zwp_fullscreen_shell_v1; }
        static zwp_fullscreen_shell_v1 *fromObject(struct ::zwp_fullscreen_shell_v1 *object);

        bool isInitialized() const;

        static const struct ::wl_interface *interface();

        enum capability {
            capability_arbitrary_modes = 1, // compositor is capable of almost any output mode
            capability_cursor_plane = 2, // compositor has a separate cursor plane
        };

        enum present_method {
            present_method_default = 0, // no preference, apply default policy
            present_method_center = 1, // center the surface on the output
            present_method_zoom = 2, // scale the surface, preserving aspect ratio, to the largest size that will fit on the output
            present_method_zoom_crop = 3, // scale the surface, preserving aspect ratio, to fully fill the output cropping if needed
            present_method_stretch = 4, // scale the surface to the size of the output ignoring aspect ratio
        };

        enum error {
            error_invalid_method = 0, // present_method is not known
        };

        void release();
        void present_surface(struct ::wl_surface *surface, uint32_t method, struct ::wl_output *output);
        struct ::zwp_fullscreen_shell_mode_feedback_v1 *present_surface_for_mode(struct ::wl_surface *surface, struct ::wl_output *output, int32_t framerate);

    protected:
        virtual void zwp_fullscreen_shell_v1_capability(uint32_t capability);

    private:
        void init_listener();
        static const struct zwp_fullscreen_shell_v1_listener m_zwp_fullscreen_shell_v1_listener;
        static void handle_capability(
            void *data,
            struct ::zwp_fullscreen_shell_v1 *object,
            uint32_t capability);
        struct ::zwp_fullscreen_shell_v1 *m_zwp_fullscreen_shell_v1;
    };

    class  zwp_fullscreen_shell_mode_feedback_v1
    {
    public:
        zwp_fullscreen_shell_mode_feedback_v1(struct ::wl_registry *registry, int id, int version);
        zwp_fullscreen_shell_mode_feedback_v1(struct ::zwp_fullscreen_shell_mode_feedback_v1 *object);
        zwp_fullscreen_shell_mode_feedback_v1();

        virtual ~zwp_fullscreen_shell_mode_feedback_v1();

        void init(struct ::wl_registry *registry, int id, int version);
        void init(struct ::zwp_fullscreen_shell_mode_feedback_v1 *object);

        struct ::zwp_fullscreen_shell_mode_feedback_v1 *object() { return m_zwp_fullscreen_shell_mode_feedback_v1; }
        const struct ::zwp_fullscreen_shell_mode_feedback_v1 *object() const { return m_zwp_fullscreen_shell_mode_feedback_v1; }
        static zwp_fullscreen_shell_mode_feedback_v1 *fromObject(struct ::zwp_fullscreen_shell_mode_feedback_v1 *object);

        bool isInitialized() const;

        static const struct ::wl_interface *interface();

    protected:
        virtual void zwp_fullscreen_shell_mode_feedback_v1_mode_successful();
        virtual void zwp_fullscreen_shell_mode_feedback_v1_mode_failed();
        virtual void zwp_fullscreen_shell_mode_feedback_v1_present_cancelled();

    private:
        void init_listener();
        static const struct zwp_fullscreen_shell_mode_feedback_v1_listener m_zwp_fullscreen_shell_mode_feedback_v1_listener;
        static void handle_mode_successful(
            void *data,
            struct ::zwp_fullscreen_shell_mode_feedback_v1 *object);
        static void handle_mode_failed(
            void *data,
            struct ::zwp_fullscreen_shell_mode_feedback_v1 *object);
        static void handle_present_cancelled(
            void *data,
            struct ::zwp_fullscreen_shell_mode_feedback_v1 *object);
        struct ::zwp_fullscreen_shell_mode_feedback_v1 *m_zwp_fullscreen_shell_mode_feedback_v1;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
