/****************************************************************************
** Meta object code from reading C++ file 'qwaylandquickshellsurfaceitem.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/extensions/qwaylandquickshellsurfaceitem.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandquickshellsurfaceitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandQuickShellSurfaceItem_t {
    QByteArrayData data[10];
    char stringdata0[172];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandQuickShellSurfaceItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandQuickShellSurfaceItem_t qt_meta_stringdata_QWaylandQuickShellSurfaceItem = {
    {
QT_MOC_LITERAL(0, 0, 29), // "QWaylandQuickShellSurfaceItem"
QT_MOC_LITERAL(1, 30, 19), // "shellSurfaceChanged"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 15), // "moveItemChanged"
QT_MOC_LITERAL(4, 67, 27), // "autoCreatePopupItemsChanged"
QT_MOC_LITERAL(5, 95, 12), // "shellSurface"
QT_MOC_LITERAL(6, 108, 21), // "QWaylandShellSurface*"
QT_MOC_LITERAL(7, 130, 8), // "moveItem"
QT_MOC_LITERAL(8, 139, 11), // "QQuickItem*"
QT_MOC_LITERAL(9, 151, 20) // "autoCreatePopupItems"

    },
    "QWaylandQuickShellSurfaceItem\0"
    "shellSurfaceChanged\0\0moveItemChanged\0"
    "autoCreatePopupItemsChanged\0shellSurface\0"
    "QWaylandShellSurface*\0moveItem\0"
    "QQuickItem*\0autoCreatePopupItems"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandQuickShellSurfaceItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       3,   32, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,
       3,    0,   30,    2, 0x06 /* Public */,
       4,    0,   31,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       5, 0x80000000 | 6, 0x0049510b,
       7, 0x80000000 | 8, 0x0049510b,
       9, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       1,
       2,

       0        // eod
};

void QWaylandQuickShellSurfaceItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandQuickShellSurfaceItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->shellSurfaceChanged(); break;
        case 1: _t->moveItemChanged(); break;
        case 2: _t->autoCreatePopupItemsChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandQuickShellSurfaceItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickShellSurfaceItem::shellSurfaceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickShellSurfaceItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickShellSurfaceItem::moveItemChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandQuickShellSurfaceItem::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandQuickShellSurfaceItem::autoCreatePopupItemsChanged)) {
                *result = 2;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandQuickShellSurfaceItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QWaylandShellSurface**>(_v) = _t->shellSurface(); break;
        case 1: *reinterpret_cast< QQuickItem**>(_v) = _t->moveItem(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->autoCreatePopupItems(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QWaylandQuickShellSurfaceItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setShellSurface(*reinterpret_cast< QWaylandShellSurface**>(_v)); break;
        case 1: _t->setMoveItem(*reinterpret_cast< QQuickItem**>(_v)); break;
        case 2: _t->setAutoCreatePopupItems(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandQuickShellSurfaceItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandQuickItem::staticMetaObject>(),
    qt_meta_stringdata_QWaylandQuickShellSurfaceItem.data,
    qt_meta_data_QWaylandQuickShellSurfaceItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandQuickShellSurfaceItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandQuickShellSurfaceItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandQuickShellSurfaceItem.stringdata0))
        return static_cast<void*>(this);
    return QWaylandQuickItem::qt_metacast(_clname);
}

int QWaylandQuickShellSurfaceItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandQuickShellSurfaceItem::shellSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandQuickShellSurfaceItem::moveItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWaylandQuickShellSurfaceItem::autoCreatePopupItemsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
