/****************************************************************************
** Meta object code from reading C++ file 'qwaylandview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/compositor_api/qwaylandview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandView_t {
    QByteArrayData data[14];
    char stringdata0[209];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandView_t qt_meta_stringdata_QWaylandView = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QWaylandView"
QT_MOC_LITERAL(1, 13, 14), // "surfaceChanged"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 16), // "surfaceDestroyed"
QT_MOC_LITERAL(4, 46, 13), // "outputChanged"
QT_MOC_LITERAL(5, 60, 19), // "bufferLockedChanged"
QT_MOC_LITERAL(6, 80, 30), // "allowDiscardFrontBufferChanged"
QT_MOC_LITERAL(7, 111, 12), // "renderObject"
QT_MOC_LITERAL(8, 124, 7), // "surface"
QT_MOC_LITERAL(9, 132, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(10, 149, 6), // "output"
QT_MOC_LITERAL(11, 156, 15), // "QWaylandOutput*"
QT_MOC_LITERAL(12, 172, 12), // "bufferLocked"
QT_MOC_LITERAL(13, 185, 23) // "allowDiscardFrontBuffer"

    },
    "QWaylandView\0surfaceChanged\0\0"
    "surfaceDestroyed\0outputChanged\0"
    "bufferLockedChanged\0allowDiscardFrontBufferChanged\0"
    "renderObject\0surface\0QWaylandSurface*\0"
    "output\0QWaylandOutput*\0bufferLocked\0"
    "allowDiscardFrontBuffer"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       5,   14, // methods
       5,   44, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   39,    2, 0x06 /* Public */,
       3,    0,   40,    2, 0x06 /* Public */,
       4,    0,   41,    2, 0x06 /* Public */,
       5,    0,   42,    2, 0x06 /* Public */,
       6,    0,   43,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
       7, QMetaType::QObjectStar, 0x00095401,
       8, 0x80000000 | 9, 0x0049510b,
      10, 0x80000000 | 11, 0x0049510b,
      12, QMetaType::Bool, 0x00495103,
      13, QMetaType::Bool, 0x00495103,

 // properties: notify_signal_id
       0,
       0,
       2,
       3,
       4,

       0        // eod
};

void QWaylandView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->surfaceChanged(); break;
        case 1: _t->surfaceDestroyed(); break;
        case 2: _t->outputChanged(); break;
        case 3: _t->bufferLockedChanged(); break;
        case 4: _t->allowDiscardFrontBufferChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandView::surfaceChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandView::surfaceDestroyed)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandView::outputChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandView::bufferLockedChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandView::allowDiscardFrontBufferChanged)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandSurface* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QObject**>(_v) = _t->renderObject(); break;
        case 1: *reinterpret_cast< QWaylandSurface**>(_v) = _t->surface(); break;
        case 2: *reinterpret_cast< QWaylandOutput**>(_v) = _t->output(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->isBufferLocked(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->allowDiscardFrontBuffer(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QWaylandView *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSurface(*reinterpret_cast< QWaylandSurface**>(_v)); break;
        case 2: _t->setOutput(*reinterpret_cast< QWaylandOutput**>(_v)); break;
        case 3: _t->setBufferLocked(*reinterpret_cast< bool*>(_v)); break;
        case 4: _t->setAllowDiscardFrontBuffer(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandView::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QWaylandView.data,
    qt_meta_data_QWaylandView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandView.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QWaylandView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 5;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandView::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandView::surfaceDestroyed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWaylandView::outputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QWaylandView::bufferLockedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QWaylandView::allowDiscardFrontBufferChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
