/****************************************************************************
** Meta object code from reading C++ file 'qwaylandxdgshell.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/compositor/extensions/qwaylandxdgshell.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qwaylandxdgshell.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QWaylandXdgShell_t {
    QByteArrayData data[24];
    char stringdata0[296];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandXdgShell_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandXdgShell_t qt_meta_stringdata_QWaylandXdgShell = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QWaylandXdgShell"
QT_MOC_LITERAL(1, 17, 17), // "xdgSurfaceCreated"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 19), // "QWaylandXdgSurface*"
QT_MOC_LITERAL(4, 56, 10), // "xdgSurface"
QT_MOC_LITERAL(5, 67, 15), // "toplevelCreated"
QT_MOC_LITERAL(6, 83, 20), // "QWaylandXdgToplevel*"
QT_MOC_LITERAL(7, 104, 8), // "toplevel"
QT_MOC_LITERAL(8, 113, 12), // "popupCreated"
QT_MOC_LITERAL(9, 126, 17), // "QWaylandXdgPopup*"
QT_MOC_LITERAL(10, 144, 5), // "popup"
QT_MOC_LITERAL(11, 150, 4), // "pong"
QT_MOC_LITERAL(12, 155, 6), // "serial"
QT_MOC_LITERAL(13, 162, 4), // "ping"
QT_MOC_LITERAL(14, 167, 15), // "QWaylandClient*"
QT_MOC_LITERAL(15, 183, 6), // "client"
QT_MOC_LITERAL(16, 190, 17), // "handleSeatChanged"
QT_MOC_LITERAL(17, 208, 13), // "QWaylandSeat*"
QT_MOC_LITERAL(18, 222, 7), // "newSeat"
QT_MOC_LITERAL(19, 230, 7), // "oldSeat"
QT_MOC_LITERAL(20, 238, 18), // "handleFocusChanged"
QT_MOC_LITERAL(21, 257, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(22, 274, 10), // "newSurface"
QT_MOC_LITERAL(23, 285, 10) // "oldSurface"

    },
    "QWaylandXdgShell\0xdgSurfaceCreated\0\0"
    "QWaylandXdgSurface*\0xdgSurface\0"
    "toplevelCreated\0QWaylandXdgToplevel*\0"
    "toplevel\0popupCreated\0QWaylandXdgPopup*\0"
    "popup\0pong\0serial\0ping\0QWaylandClient*\0"
    "client\0handleSeatChanged\0QWaylandSeat*\0"
    "newSeat\0oldSeat\0handleFocusChanged\0"
    "QWaylandSurface*\0newSurface\0oldSurface"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandXdgShell[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   49,    2, 0x06 /* Public */,
       5,    2,   52,    2, 0x06 /* Public */,
       8,    2,   57,    2, 0x06 /* Public */,
      11,    1,   62,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    1,   65,    2, 0x0a /* Public */,
      16,    2,   68,    2, 0x08 /* Private */,
      20,    2,   73,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 3,    7,    4,
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 3,   10,    4,
    QMetaType::Void, QMetaType::UInt,   12,

 // slots: parameters
    QMetaType::UInt, 0x80000000 | 14,   15,
    QMetaType::Void, 0x80000000 | 17, 0x80000000 | 17,   18,   19,
    QMetaType::Void, 0x80000000 | 21, 0x80000000 | 21,   22,   23,

       0        // eod
};

void QWaylandXdgShell::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandXdgShell *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->xdgSurfaceCreated((*reinterpret_cast< QWaylandXdgSurface*(*)>(_a[1]))); break;
        case 1: _t->toplevelCreated((*reinterpret_cast< QWaylandXdgToplevel*(*)>(_a[1])),(*reinterpret_cast< QWaylandXdgSurface*(*)>(_a[2]))); break;
        case 2: _t->popupCreated((*reinterpret_cast< QWaylandXdgPopup*(*)>(_a[1])),(*reinterpret_cast< QWaylandXdgSurface*(*)>(_a[2]))); break;
        case 3: _t->pong((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 4: { uint _r = _t->ping((*reinterpret_cast< QWaylandClient*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->handleSeatChanged((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< QWaylandSeat*(*)>(_a[2]))); break;
        case 6: _t->handleFocusChanged((*reinterpret_cast< QWaylandSurface*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgSurface* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgSurface* >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgToplevel* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgPopup* >(); break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgSurface* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandXdgShell::*)(QWaylandXdgSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgShell::xdgSurfaceCreated)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgShell::*)(QWaylandXdgToplevel * , QWaylandXdgSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgShell::toplevelCreated)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgShell::*)(QWaylandXdgPopup * , QWaylandXdgSurface * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgShell::popupCreated)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgShell::*)(uint );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgShell::pong)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QWaylandXdgShell::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandShellTemplate<QWaylandXdgShell>::staticMetaObject>(),
    qt_meta_stringdata_QWaylandXdgShell.data,
    qt_meta_data_QWaylandXdgShell,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandXdgShell::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandXdgShell::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandXdgShell.stringdata0))
        return static_cast<void*>(this);
    return QWaylandShellTemplate<QWaylandXdgShell>::qt_metacast(_clname);
}

int QWaylandXdgShell::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandShellTemplate<QWaylandXdgShell>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QWaylandXdgShell::xdgSurfaceCreated(QWaylandXdgSurface * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QWaylandXdgShell::toplevelCreated(QWaylandXdgToplevel * _t1, QWaylandXdgSurface * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QWaylandXdgShell::popupCreated(QWaylandXdgPopup * _t1, QWaylandXdgSurface * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QWaylandXdgShell::pong(uint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
struct qt_meta_stringdata_QWaylandXdgSurface_t {
    QByteArrayData data[25];
    char stringdata0[360];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandXdgSurface_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandXdgSurface_t qt_meta_stringdata_QWaylandXdgSurface = {
    {
QT_MOC_LITERAL(0, 0, 18), // "QWaylandXdgSurface"
QT_MOC_LITERAL(1, 19, 15), // "DefaultProperty"
QT_MOC_LITERAL(2, 35, 4), // "data"
QT_MOC_LITERAL(3, 40, 12), // "shellChanged"
QT_MOC_LITERAL(4, 53, 0), // ""
QT_MOC_LITERAL(5, 54, 14), // "surfaceChanged"
QT_MOC_LITERAL(6, 69, 15), // "toplevelCreated"
QT_MOC_LITERAL(7, 85, 12), // "popupCreated"
QT_MOC_LITERAL(8, 98, 21), // "windowGeometryChanged"
QT_MOC_LITERAL(9, 120, 24), // "handleSurfaceSizeChanged"
QT_MOC_LITERAL(10, 145, 24), // "handleBufferScaleChanged"
QT_MOC_LITERAL(11, 170, 10), // "initialize"
QT_MOC_LITERAL(12, 181, 17), // "QWaylandXdgShell*"
QT_MOC_LITERAL(13, 199, 8), // "xdgShell"
QT_MOC_LITERAL(14, 208, 16), // "QWaylandSurface*"
QT_MOC_LITERAL(15, 225, 7), // "surface"
QT_MOC_LITERAL(16, 233, 16), // "QWaylandResource"
QT_MOC_LITERAL(17, 250, 8), // "resource"
QT_MOC_LITERAL(18, 259, 25), // "QQmlListProperty<QObject>"
QT_MOC_LITERAL(19, 285, 5), // "shell"
QT_MOC_LITERAL(20, 291, 8), // "toplevel"
QT_MOC_LITERAL(21, 300, 20), // "QWaylandXdgToplevel*"
QT_MOC_LITERAL(22, 321, 5), // "popup"
QT_MOC_LITERAL(23, 327, 17), // "QWaylandXdgPopup*"
QT_MOC_LITERAL(24, 345, 14) // "windowGeometry"

    },
    "QWaylandXdgSurface\0DefaultProperty\0"
    "data\0shellChanged\0\0surfaceChanged\0"
    "toplevelCreated\0popupCreated\0"
    "windowGeometryChanged\0handleSurfaceSizeChanged\0"
    "handleBufferScaleChanged\0initialize\0"
    "QWaylandXdgShell*\0xdgShell\0QWaylandSurface*\0"
    "surface\0QWaylandResource\0resource\0"
    "QQmlListProperty<QObject>\0shell\0"
    "toplevel\0QWaylandXdgToplevel*\0popup\0"
    "QWaylandXdgPopup*\0windowGeometry"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandXdgSurface[] = {

 // content:
       8,       // revision
       0,       // classname
       1,   14, // classinfo
       8,   16, // methods
       6,   70, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // classinfo: key, value
       1,    2,

 // signals: name, argc, parameters, tag, flags
       3,    0,   56,    4, 0x06 /* Public */,
       5,    0,   57,    4, 0x06 /* Public */,
       6,    0,   58,    4, 0x06 /* Public */,
       7,    0,   59,    4, 0x06 /* Public */,
       8,    0,   60,    4, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    0,   61,    4, 0x08 /* Private */,
      10,    0,   62,    4, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      11,    3,   63,    4, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, 0x80000000 | 12, 0x80000000 | 14, 0x80000000 | 16,   13,   15,   17,

 // properties: name, type, flags
       2, 0x80000000 | 18, 0x00094009,
      19, 0x80000000 | 12, 0x00495009,
      15, 0x80000000 | 14, 0x00495009,
      20, 0x80000000 | 21, 0x00495009,
      22, 0x80000000 | 23, 0x00495009,
      24, QMetaType::QRect, 0x00495001,

 // properties: notify_signal_id
       0,
       0,
       1,
       2,
       3,
       4,

       0        // eod
};

void QWaylandXdgSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandXdgSurface *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->shellChanged(); break;
        case 1: _t->surfaceChanged(); break;
        case 2: _t->toplevelCreated(); break;
        case 3: _t->popupCreated(); break;
        case 4: _t->windowGeometryChanged(); break;
        case 5: _t->handleSurfaceSizeChanged(); break;
        case 6: _t->handleBufferScaleChanged(); break;
        case 7: _t->initialize((*reinterpret_cast< QWaylandXdgShell*(*)>(_a[1])),(*reinterpret_cast< QWaylandSurface*(*)>(_a[2])),(*reinterpret_cast< const QWaylandResource(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 2:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandResource >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgShell* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandXdgSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurface::shellChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurface::surfaceChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurface::toplevelCreated)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurface::popupCreated)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgSurface::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgSurface::windowGeometryChanged)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgPopup* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgShell* >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgToplevel* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandXdgSurface *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QQmlListProperty<QObject>*>(_v) = _t->data(); break;
        case 1: *reinterpret_cast< QWaylandXdgShell**>(_v) = _t->shell(); break;
        case 2: *reinterpret_cast< QWaylandSurface**>(_v) = _t->surface(); break;
        case 3: *reinterpret_cast< QWaylandXdgToplevel**>(_v) = _t->toplevel(); break;
        case 4: *reinterpret_cast< QWaylandXdgPopup**>(_v) = _t->popup(); break;
        case 5: *reinterpret_cast< QRect*>(_v) = _t->windowGeometry(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandXdgSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<QWaylandShellSurfaceTemplate<QWaylandXdgSurface>::staticMetaObject>(),
    qt_meta_stringdata_QWaylandXdgSurface.data,
    qt_meta_data_QWaylandXdgSurface,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandXdgSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandXdgSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandXdgSurface.stringdata0))
        return static_cast<void*>(this);
    return QWaylandShellSurfaceTemplate<QWaylandXdgSurface>::qt_metacast(_clname);
}

int QWaylandXdgSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWaylandShellSurfaceTemplate<QWaylandXdgSurface>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 6;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 6;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandXdgSurface::shellChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandXdgSurface::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWaylandXdgSurface::toplevelCreated()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QWaylandXdgSurface::popupCreated()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QWaylandXdgSurface::windowGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
struct qt_meta_stringdata_QWaylandXdgToplevel_t {
    QByteArrayData data[61];
    char stringdata0[801];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandXdgToplevel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandXdgToplevel_t qt_meta_stringdata_QWaylandXdgToplevel = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QWaylandXdgToplevel"
QT_MOC_LITERAL(1, 20, 21), // "parentToplevelChanged"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 12), // "titleChanged"
QT_MOC_LITERAL(4, 56, 12), // "appIdChanged"
QT_MOC_LITERAL(5, 69, 14), // "maxSizeChanged"
QT_MOC_LITERAL(6, 84, 14), // "minSizeChanged"
QT_MOC_LITERAL(7, 99, 9), // "startMove"
QT_MOC_LITERAL(8, 109, 13), // "QWaylandSeat*"
QT_MOC_LITERAL(9, 123, 4), // "seat"
QT_MOC_LITERAL(10, 128, 11), // "startResize"
QT_MOC_LITERAL(11, 140, 9), // "Qt::Edges"
QT_MOC_LITERAL(12, 150, 5), // "edges"
QT_MOC_LITERAL(13, 156, 13), // "statesChanged"
QT_MOC_LITERAL(14, 170, 16), // "maximizedChanged"
QT_MOC_LITERAL(15, 187, 17), // "fullscreenChanged"
QT_MOC_LITERAL(16, 205, 15), // "resizingChanged"
QT_MOC_LITERAL(17, 221, 16), // "activatedChanged"
QT_MOC_LITERAL(18, 238, 14), // "showWindowMenu"
QT_MOC_LITERAL(19, 253, 20), // "localSurfacePosition"
QT_MOC_LITERAL(20, 274, 12), // "setMaximized"
QT_MOC_LITERAL(21, 287, 14), // "unsetMaximized"
QT_MOC_LITERAL(22, 302, 13), // "setFullscreen"
QT_MOC_LITERAL(23, 316, 15), // "QWaylandOutput*"
QT_MOC_LITERAL(24, 332, 6), // "output"
QT_MOC_LITERAL(25, 339, 15), // "unsetFullscreen"
QT_MOC_LITERAL(26, 355, 12), // "setMinimized"
QT_MOC_LITERAL(27, 368, 21), // "decorationModeChanged"
QT_MOC_LITERAL(28, 390, 13), // "sizeForResize"
QT_MOC_LITERAL(29, 404, 4), // "size"
QT_MOC_LITERAL(30, 409, 5), // "delta"
QT_MOC_LITERAL(31, 415, 13), // "sendConfigure"
QT_MOC_LITERAL(32, 429, 12), // "QVector<int>"
QT_MOC_LITERAL(33, 442, 6), // "states"
QT_MOC_LITERAL(34, 449, 9), // "sendClose"
QT_MOC_LITERAL(35, 459, 13), // "sendMaximized"
QT_MOC_LITERAL(36, 473, 15), // "sendUnmaximized"
QT_MOC_LITERAL(37, 489, 14), // "sendFullscreen"
QT_MOC_LITERAL(38, 504, 12), // "sendResizing"
QT_MOC_LITERAL(39, 517, 7), // "maxSize"
QT_MOC_LITERAL(40, 525, 10), // "xdgSurface"
QT_MOC_LITERAL(41, 536, 19), // "QWaylandXdgSurface*"
QT_MOC_LITERAL(42, 556, 14), // "parentToplevel"
QT_MOC_LITERAL(43, 571, 20), // "QWaylandXdgToplevel*"
QT_MOC_LITERAL(44, 592, 5), // "title"
QT_MOC_LITERAL(45, 598, 5), // "appId"
QT_MOC_LITERAL(46, 604, 7), // "minSize"
QT_MOC_LITERAL(47, 612, 10), // "QList<int>"
QT_MOC_LITERAL(48, 623, 9), // "maximized"
QT_MOC_LITERAL(49, 633, 10), // "fullscreen"
QT_MOC_LITERAL(50, 644, 8), // "resizing"
QT_MOC_LITERAL(51, 653, 9), // "activated"
QT_MOC_LITERAL(52, 663, 14), // "decorationMode"
QT_MOC_LITERAL(53, 678, 14), // "DecorationMode"
QT_MOC_LITERAL(54, 693, 5), // "State"
QT_MOC_LITERAL(55, 699, 14), // "MaximizedState"
QT_MOC_LITERAL(56, 714, 15), // "FullscreenState"
QT_MOC_LITERAL(57, 730, 13), // "ResizingState"
QT_MOC_LITERAL(58, 744, 14), // "ActivatedState"
QT_MOC_LITERAL(59, 759, 20), // "ClientSideDecoration"
QT_MOC_LITERAL(60, 780, 20) // "ServerSideDecoration"

    },
    "QWaylandXdgToplevel\0parentToplevelChanged\0"
    "\0titleChanged\0appIdChanged\0maxSizeChanged\0"
    "minSizeChanged\0startMove\0QWaylandSeat*\0"
    "seat\0startResize\0Qt::Edges\0edges\0"
    "statesChanged\0maximizedChanged\0"
    "fullscreenChanged\0resizingChanged\0"
    "activatedChanged\0showWindowMenu\0"
    "localSurfacePosition\0setMaximized\0"
    "unsetMaximized\0setFullscreen\0"
    "QWaylandOutput*\0output\0unsetFullscreen\0"
    "setMinimized\0decorationModeChanged\0"
    "sizeForResize\0size\0delta\0sendConfigure\0"
    "QVector<int>\0states\0sendClose\0"
    "sendMaximized\0sendUnmaximized\0"
    "sendFullscreen\0sendResizing\0maxSize\0"
    "xdgSurface\0QWaylandXdgSurface*\0"
    "parentToplevel\0QWaylandXdgToplevel*\0"
    "title\0appId\0minSize\0QList<int>\0maximized\0"
    "fullscreen\0resizing\0activated\0"
    "decorationMode\0DecorationMode\0State\0"
    "MaximizedState\0FullscreenState\0"
    "ResizingState\0ActivatedState\0"
    "ClientSideDecoration\0ServerSideDecoration"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandXdgToplevel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      27,   14, // methods
      12,  206, // properties
       2,  254, // enums/sets
       0,    0, // constructors
       0,       // flags
      19,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  149,    2, 0x06 /* Public */,
       3,    0,  150,    2, 0x06 /* Public */,
       4,    0,  151,    2, 0x06 /* Public */,
       5,    0,  152,    2, 0x06 /* Public */,
       6,    0,  153,    2, 0x06 /* Public */,
       7,    1,  154,    2, 0x06 /* Public */,
      10,    2,  157,    2, 0x06 /* Public */,
      13,    0,  162,    2, 0x06 /* Public */,
      14,    0,  163,    2, 0x06 /* Public */,
      15,    0,  164,    2, 0x06 /* Public */,
      16,    0,  165,    2, 0x06 /* Public */,
      17,    0,  166,    2, 0x06 /* Public */,
      18,    2,  167,    2, 0x06 /* Public */,
      20,    0,  172,    2, 0x06 /* Public */,
      21,    0,  173,    2, 0x06 /* Public */,
      22,    1,  174,    2, 0x06 /* Public */,
      25,    0,  177,    2, 0x06 /* Public */,
      26,    0,  178,    2, 0x06 /* Public */,
      27,    0,  179,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      28,    3,  180,    2, 0x02 /* Public */,
      31,    2,  187,    2, 0x02 /* Public */,
      34,    0,  192,    2, 0x02 /* Public */,
      35,    1,  193,    2, 0x02 /* Public */,
      36,    1,  196,    2, 0x02 /* Public */,
      36,    0,  199,    2, 0x22 /* Public | MethodCloned */,
      37,    1,  200,    2, 0x02 /* Public */,
      38,    1,  203,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 8, 0x80000000 | 11,    9,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8, QMetaType::QPoint,    9,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 23,   24,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::QSize, QMetaType::QSizeF, QMetaType::QPointF, 0x80000000 | 11,   29,   30,   12,
    QMetaType::UInt, QMetaType::QSize, 0x80000000 | 32,   29,   33,
    QMetaType::Void,
    QMetaType::UInt, QMetaType::QSize,   29,
    QMetaType::UInt, QMetaType::QSize,   29,
    QMetaType::UInt,
    QMetaType::UInt, QMetaType::QSize,   29,
    QMetaType::UInt, QMetaType::QSize,   39,

 // properties: name, type, flags
      40, 0x80000000 | 41, 0x00095409,
      42, 0x80000000 | 43, 0x00495009,
      44, QMetaType::QString, 0x00495001,
      45, QMetaType::QString, 0x00495001,
      39, QMetaType::QSize, 0x00495001,
      46, QMetaType::QSize, 0x00495001,
      33, 0x80000000 | 47, 0x00495009,
      48, QMetaType::Bool, 0x00495001,
      49, QMetaType::Bool, 0x00495001,
      50, QMetaType::Bool, 0x00495001,
      51, QMetaType::Bool, 0x00495001,
      52, 0x80000000 | 53, 0x00495009,

 // properties: notify_signal_id
       0,
       0,
       1,
       2,
       3,
       4,
       7,
       8,
       9,
      10,
      11,
      18,

 // enums: name, alias, flags, count, data
      54,   54, 0x0,    4,  264,
      53,   53, 0x0,    2,  272,

 // enum data: key, value
      55, uint(QWaylandXdgToplevel::MaximizedState),
      56, uint(QWaylandXdgToplevel::FullscreenState),
      57, uint(QWaylandXdgToplevel::ResizingState),
      58, uint(QWaylandXdgToplevel::ActivatedState),
      59, uint(QWaylandXdgToplevel::ClientSideDecoration),
      60, uint(QWaylandXdgToplevel::ServerSideDecoration),

       0        // eod
};

void QWaylandXdgToplevel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandXdgToplevel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->parentToplevelChanged(); break;
        case 1: _t->titleChanged(); break;
        case 2: _t->appIdChanged(); break;
        case 3: _t->maxSizeChanged(); break;
        case 4: _t->minSizeChanged(); break;
        case 5: _t->startMove((*reinterpret_cast< QWaylandSeat*(*)>(_a[1]))); break;
        case 6: _t->startResize((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< Qt::Edges(*)>(_a[2]))); break;
        case 7: _t->statesChanged(); break;
        case 8: _t->maximizedChanged(); break;
        case 9: _t->fullscreenChanged(); break;
        case 10: _t->resizingChanged(); break;
        case 11: _t->activatedChanged(); break;
        case 12: _t->showWindowMenu((*reinterpret_cast< QWaylandSeat*(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2]))); break;
        case 13: _t->setMaximized(); break;
        case 14: _t->unsetMaximized(); break;
        case 15: _t->setFullscreen((*reinterpret_cast< QWaylandOutput*(*)>(_a[1]))); break;
        case 16: _t->unsetFullscreen(); break;
        case 17: _t->setMinimized(); break;
        case 18: _t->decorationModeChanged(); break;
        case 19: { QSize _r = _t->sizeForResize((*reinterpret_cast< const QSizeF(*)>(_a[1])),(*reinterpret_cast< const QPointF(*)>(_a[2])),(*reinterpret_cast< Qt::Edges(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QSize*>(_a[0]) = std::move(_r); }  break;
        case 20: { uint _r = _t->sendConfigure((*reinterpret_cast< const QSize(*)>(_a[1])),(*reinterpret_cast< const QVector<int>(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->sendClose(); break;
        case 22: { uint _r = _t->sendMaximized((*reinterpret_cast< const QSize(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 23: { uint _r = _t->sendUnmaximized((*reinterpret_cast< const QSize(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 24: { uint _r = _t->sendUnmaximized();
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 25: { uint _r = _t->sendFullscreen((*reinterpret_cast< const QSize(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 26: { uint _r = _t->sendResizing((*reinterpret_cast< const QSize(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<int> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::parentToplevelChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::titleChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::appIdChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::maxSizeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::minSizeChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)(QWaylandSeat * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::startMove)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)(QWaylandSeat * , Qt::Edges );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::startResize)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::statesChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::maximizedChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::fullscreenChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::resizingChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::activatedChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)(QWaylandSeat * , const QPoint & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::showWindowMenu)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::setMaximized)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::unsetMaximized)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)(QWaylandOutput * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::setFullscreen)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::unsetFullscreen)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::setMinimized)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (QWaylandXdgToplevel::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgToplevel::decorationModeChanged)) {
                *result = 18;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgSurface* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgToplevel* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandXdgToplevel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QWaylandXdgSurface**>(_v) = _t->xdgSurface(); break;
        case 1: *reinterpret_cast< QWaylandXdgToplevel**>(_v) = _t->parentToplevel(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->title(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->appId(); break;
        case 4: *reinterpret_cast< QSize*>(_v) = _t->maxSize(); break;
        case 5: *reinterpret_cast< QSize*>(_v) = _t->minSize(); break;
        case 6: *reinterpret_cast< QList<int>*>(_v) = _t->statesAsInts(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->maximized(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->fullscreen(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->resizing(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->activated(); break;
        case 11: *reinterpret_cast< DecorationMode*>(_v) = _t->decorationMode(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandXdgToplevel::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QWaylandXdgToplevel.data,
    qt_meta_data_QWaylandXdgToplevel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandXdgToplevel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandXdgToplevel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandXdgToplevel.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QWaylandXdgToplevel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 12;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandXdgToplevel::parentToplevelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QWaylandXdgToplevel::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QWaylandXdgToplevel::appIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QWaylandXdgToplevel::maxSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QWaylandXdgToplevel::minSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QWaylandXdgToplevel::startMove(QWaylandSeat * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QWaylandXdgToplevel::startResize(QWaylandSeat * _t1, Qt::Edges _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QWaylandXdgToplevel::statesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QWaylandXdgToplevel::maximizedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QWaylandXdgToplevel::fullscreenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QWaylandXdgToplevel::resizingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QWaylandXdgToplevel::activatedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QWaylandXdgToplevel::showWindowMenu(QWaylandSeat * _t1, const QPoint & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QWaylandXdgToplevel::setMaximized()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QWaylandXdgToplevel::unsetMaximized()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QWaylandXdgToplevel::setFullscreen(QWaylandOutput * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void QWaylandXdgToplevel::unsetFullscreen()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QWaylandXdgToplevel::setMinimized()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QWaylandXdgToplevel::decorationModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}
struct qt_meta_stringdata_QWaylandXdgPopup_t {
    QByteArrayData data[21];
    char stringdata0[306];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QWaylandXdgPopup_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QWaylandXdgPopup_t qt_meta_stringdata_QWaylandXdgPopup = {
    {
QT_MOC_LITERAL(0, 0, 16), // "QWaylandXdgPopup"
QT_MOC_LITERAL(1, 17, 25), // "configuredGeometryChanged"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 13), // "sendConfigure"
QT_MOC_LITERAL(4, 58, 8), // "geometry"
QT_MOC_LITERAL(5, 67, 13), // "sendPopupDone"
QT_MOC_LITERAL(6, 81, 10), // "xdgSurface"
QT_MOC_LITERAL(7, 92, 19), // "QWaylandXdgSurface*"
QT_MOC_LITERAL(8, 112, 16), // "parentXdgSurface"
QT_MOC_LITERAL(9, 129, 18), // "configuredGeometry"
QT_MOC_LITERAL(10, 148, 10), // "anchorRect"
QT_MOC_LITERAL(11, 159, 11), // "anchorEdges"
QT_MOC_LITERAL(12, 171, 9), // "Qt::Edges"
QT_MOC_LITERAL(13, 181, 12), // "gravityEdges"
QT_MOC_LITERAL(14, 194, 16), // "slideConstraints"
QT_MOC_LITERAL(15, 211, 16), // "Qt::Orientations"
QT_MOC_LITERAL(16, 228, 15), // "flipConstraints"
QT_MOC_LITERAL(17, 244, 17), // "resizeConstraints"
QT_MOC_LITERAL(18, 262, 6), // "offset"
QT_MOC_LITERAL(19, 269, 14), // "positionerSize"
QT_MOC_LITERAL(20, 284, 21) // "unconstrainedPosition"

    },
    "QWaylandXdgPopup\0configuredGeometryChanged\0"
    "\0sendConfigure\0geometry\0sendPopupDone\0"
    "xdgSurface\0QWaylandXdgSurface*\0"
    "parentXdgSurface\0configuredGeometry\0"
    "anchorRect\0anchorEdges\0Qt::Edges\0"
    "gravityEdges\0slideConstraints\0"
    "Qt::Orientations\0flipConstraints\0"
    "resizeConstraints\0offset\0positionerSize\0"
    "unconstrainedPosition"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QWaylandXdgPopup[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
      12,   37, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   32,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       3,    1,   33,    2, 0x02 /* Public */,
       5,    0,   36,    2, 0x82 /* Public | MethodRevisioned */,

 // signals: revision
       0,

 // methods: revision
       0,
      14,

 // signals: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::UInt, QMetaType::QRect,    4,
    QMetaType::Void,

 // properties: name, type, flags
       6, 0x80000000 | 7, 0x00095409,
       8, 0x80000000 | 7, 0x00095409,
       9, QMetaType::QRect, 0x00495001,
      10, QMetaType::QRect, 0x00095401,
      11, 0x80000000 | 12, 0x00095409,
      13, 0x80000000 | 12, 0x00095409,
      14, 0x80000000 | 15, 0x00095409,
      16, 0x80000000 | 15, 0x00095409,
      17, 0x80000000 | 15, 0x00095409,
      18, QMetaType::QPoint, 0x00095401,
      19, QMetaType::QSize, 0x00095401,
      20, QMetaType::QPoint, 0x00095401,

 // properties: notify_signal_id
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

       0        // eod
};

void QWaylandXdgPopup::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QWaylandXdgPopup *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->configuredGeometryChanged(); break;
        case 1: { uint _r = _t->sendConfigure((*reinterpret_cast< const QRect(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = std::move(_r); }  break;
        case 2: _t->sendPopupDone(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QWaylandXdgPopup::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QWaylandXdgPopup::configuredGeometryChanged)) {
                *result = 0;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWaylandXdgSurface* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QWaylandXdgPopup *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QWaylandXdgSurface**>(_v) = _t->xdgSurface(); break;
        case 1: *reinterpret_cast< QWaylandXdgSurface**>(_v) = _t->parentXdgSurface(); break;
        case 2: *reinterpret_cast< QRect*>(_v) = _t->configuredGeometry(); break;
        case 3: *reinterpret_cast< QRect*>(_v) = _t->anchorRect(); break;
        case 4: *reinterpret_cast< Qt::Edges*>(_v) = _t->anchorEdges(); break;
        case 5: *reinterpret_cast< Qt::Edges*>(_v) = _t->gravityEdges(); break;
        case 6: *reinterpret_cast< Qt::Orientations*>(_v) = _t->slideConstraints(); break;
        case 7: *reinterpret_cast< Qt::Orientations*>(_v) = _t->flipConstraints(); break;
        case 8: *reinterpret_cast< Qt::Orientations*>(_v) = _t->resizeConstraints(); break;
        case 9: *reinterpret_cast< QPoint*>(_v) = _t->offset(); break;
        case 10: *reinterpret_cast< QSize*>(_v) = _t->positionerSize(); break;
        case 11: *reinterpret_cast< QPoint*>(_v) = _t->unconstrainedPosition(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QWaylandXdgPopup::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QWaylandXdgPopup.data,
    qt_meta_data_QWaylandXdgPopup,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QWaylandXdgPopup::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QWaylandXdgPopup::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QWaylandXdgPopup.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QWaylandXdgPopup::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 12;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 12;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QWaylandXdgPopup::configuredGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
